# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
from oslo_utils import uuidutils
import testtools

from sahara import exceptions as exc
from sahara.service import quotas
from sahara.tests.unit import base


class FakeFlavor(object):
    def __init__(self, ram, vcpu):
        self.ram = ram
        self.vcpus = vcpu


class FakeNovaClient(object):
    def __init__(self, lims):
        self.lims = lims

    @property
    def limits(self):
        return self

    def to_dict(self):
        return self.lims

    def get(self):
        return self

    @property
    def flavors(self):
        return {'id1': FakeFlavor(4, 2)}


class CinderLimit(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value


class FakeCinderClient(object):
    def __init__(self, lims):
        self.lims = lims

    @property
    def limits(self):
        return self

    @property
    def absolute(self):
        return self.lims

    def get(self):
        return self

    def to_dict(self):
        return self


class FakeNeutronClient(object):
    def __init__(self, lims):
        self.lims = lims

    def show_quota(self, tenant_id):
        return {'quota': self.lims}

    def list_floatingips(self, tenant_id):
        return {'floatingips': [1, 2, 3, 4, 5]}

    def list_security_groups(self, tenant_id):
        return {'security_groups': [1, 2, 3, 4, 5, 6, 7]}

    def list_security_group_rules(self, tenant_id):
        return {'security_groups_rules': [1, 2, 3]}

    def list_ports(self, tenant_id):
        return {'ports': []}


class FakeCluster(object):
    def __init__(self, node_groups):
        self.node_groups = node_groups


class FakeNodeGroup(object):
    def __init__(self, count, auto_sg, volumes_size, volumes_per_node, pool,
                 flavor_id, ports):
        self.count = count
        self.auto_security_group = auto_sg
        self.volumes_size = volumes_size
        self.volumes_per_node = volumes_per_node
        self.floating_ip_pool = pool
        self.flavor_id = flavor_id
        self.open_ports = ports
        self.id = uuidutils.generate_uuid()


nova_limits = {
    'absolute': {
        'maxTotalRAMSize': 10,
        'totalRAMUsed': 1,
        'maxTotalCores': 15,
        'totalCoresUsed': 5,
        'maxTotalInstances': 5,
        'totalInstancesUsed': 2,
        'maxTotalFloatingIps': 300,
        'totalFloatingIpsUsed': 100,
        'maxSecurityGroups': 50,
        'totalSecurityGroupsUsed': 22,
        'maxSecurityGroupRules': -1,  # unlimited quota test
    }
}

neutron_limits = {
    'floatingip': 2345,
    'security_group': 1523,
    'security_group_rule': 332,
    'port': -1
}

cinder_limits = [
    CinderLimit(name='maxTotalVolumes', value=5),
    CinderLimit(name='totalVolumesUsed', value=3),
    CinderLimit(name='maxTotalVolumeGigabytes', value=10),
    CinderLimit(name='totalGigabytesUsed', value=2)
]


class TestQuotas(base.SaharaTestCase):
    LIST_LIMITS = ['ram', 'cpu', 'instances', 'floatingips',
                   'security_groups', 'security_group_rules', 'ports',
                   'volumes', 'volume_gbs']

    def test_get_zero_limits(self):
        res = quotas._get_zero_limits()
        self.assertEqual(9, len(res))
        for key in self.LIST_LIMITS:
            self.assertEqual(0, res[key])

    @mock.patch('sahara.service.quotas._get_avail_limits')
    def test_check_limits(self, mock_avail_limits):
        avail_limits = {}
        req_limits = {}

        for key in self.LIST_LIMITS:
            avail_limits[key] = quotas.UNLIMITED
            req_limits[key] = 100500
        mock_avail_limits.return_value = avail_limits
        self.assertIsNone(quotas._check_limits(req_limits))

        for key in self.LIST_LIMITS:
            avail_limits[key] = 2
            req_limits[key] = 1
        mock_avail_limits.return_value = avail_limits
        self.assertIsNone(quotas._check_limits(req_limits))

        for key in self.LIST_LIMITS:
            req_limits[key] = 2
        self.assertIsNone(quotas._check_limits(req_limits))

        for key in self.LIST_LIMITS:
            req_limits[key] = 3
        self.assertRaises(exc.QuotaException, quotas._check_limits, req_limits)

    @mock.patch('sahara.utils.openstack.nova.client')
    def test_update_limits_for_ng(self, nova_mock):
        flavor_mock = mock.Mock()
        type(flavor_mock).ram = mock.PropertyMock(return_value=4)
        type(flavor_mock).vcpus = mock.PropertyMock(return_value=2)

        flavor_get_mock = mock.Mock()
        flavor_get_mock.get.return_value = flavor_mock

        type(nova_mock.return_value).flavors = mock.PropertyMock(
            return_value=flavor_get_mock)

        ng = mock.Mock()
        type(ng).flavor_id = mock.PropertyMock(return_value=3)
        type(ng).floating_ip_pool = mock.PropertyMock(return_value='pool')
        type(ng).volumes_per_node = mock.PropertyMock(return_value=4)
        type(ng).volumes_size = mock.PropertyMock(return_value=5)
        type(ng).auto_security_group = mock.PropertyMock(return_value=True)
        type(ng).open_ports = mock.PropertyMock(return_value=[1111, 2222])

        limits = quotas._get_zero_limits()
        quotas._update_limits_for_ng(limits, ng, 3)

        self.assertEqual(3, limits['instances'])
        self.assertEqual(12, limits['ram'])
        self.assertEqual(6, limits['cpu'])
        self.assertEqual(3, limits['floatingips'])
        self.assertEqual(12, limits['volumes'])
        self.assertEqual(60, limits['volume_gbs'])
        self.assertEqual(1, limits['security_groups'])
        self.assertEqual(5, limits['security_group_rules'])
        self.assertEqual(3, limits['ports'])

    @mock.patch('sahara.utils.openstack.nova.client',
                return_value=FakeNovaClient(nova_limits))
    def test_get_nova_limits(self, nova):
        self.assertEqual(
            {'cpu': 10, 'instances': 3, 'ram': 9}, quotas._get_nova_limits())

    @mock.patch('sahara.utils.openstack.cinder.client',
                return_value=FakeCinderClient(cinder_limits))
    def test_get_cinder_limits(self, cinder):
        self.assertEqual({'volumes': 2, 'volume_gbs': 8},
                         quotas._get_cinder_limits())

    @mock.patch('sahara.utils.openstack.neutron.client',
                return_value=FakeNeutronClient(neutron_limits))
    def test_neutron_limits(self, neutron):
        self.assertEqual({'floatingips': 2340,
                          'ports': 'unlimited',
                          'security_group_rules': 332,
                          'security_groups': 1516},
                         quotas._get_neutron_limits())

    @mock.patch("sahara.utils.openstack.cinder.check_cinder_exists",
                return_value=True)
    @mock.patch('sahara.utils.openstack.nova.client',
                return_value=FakeNovaClient(nova_limits))
    @mock.patch('sahara.utils.openstack.cinder.client',
                return_value=FakeCinderClient(cinder_limits))
    @mock.patch('sahara.utils.openstack.neutron.client',
                return_value=FakeNeutronClient(neutron_limits))
    def test_limits_for_cluster(self, p1, p2, p3, p4):
        ng = [FakeNodeGroup(1, False, 0, 0, None, 'id1', [1, 2, 3])]
        quotas.check_cluster(FakeCluster(ng))

        with testtools.ExpectedException(exc.QuotaException):
            quotas.check_cluster(FakeCluster([FakeNodeGroup(
                1, False, 3, 3, None, 'id1', [1, 2, 3])]))

        ng = [FakeNodeGroup(1, False, 0, 0, None, 'id1', [1, 2, 3]),
              FakeNodeGroup(0, False, 0, 0, None, 'id1', [1, 2, 3])]
        quotas.check_scaling(FakeCluster(ng), {}, {ng[1].id: 1})

        with testtools.ExpectedException(exc.QuotaException):
            quotas.check_scaling(FakeCluster(ng), {}, {ng[1].id: 3})



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pycodestyle

import re
import tokenize

from sahara.utils.hacking import commit_message
from sahara.utils.hacking import import_checks
from sahara.utils.hacking import logging_checks


RE_OSLO_IMPORTS = (re.compile(r"(((from)|(import))\s+oslo\.)"),
                   re.compile(r"(from\s+oslo\s+import)"))
RE_DICT_CONSTRUCTOR_WITH_LIST_COPY = re.compile(r".*\bdict\((\[)?(\(|\[)")
RE_USE_JSONUTILS_INVALID_LINE = re.compile(r"(import\s+json)")
RE_USE_JSONUTILS_VALID_LINE = re.compile(r"(import\s+jsonschema)")
RE_MUTABLE_DEFAULT_ARGS = re.compile(r"^\s*def .+\((.+=\{\}|.+=\[\])")


def _starts_with_any(line, *prefixes):
    for prefix in prefixes:
        if line.startswith(prefix):
            return True
    return False


def _any_in(line, *sublines):
    for subline in sublines:
        if subline in line:
            return True
    return False


def import_db_only_in_conductor(logical_line, filename):
    """Check that db calls are only in conductor, plugins module and in tests.

    S361
    """
    if _any_in(filename,
               "sahara/conductor",
               "sahara/plugins",
               "sahara/tests",
               "sahara/db"):
        return

    if _starts_with_any(logical_line,
                        "from sahara import db",
                        "from sahara.db",
                        "import sahara.db"):
        yield (0, "S361: sahara.db import only allowed in "
                  "sahara/conductor/*")


def hacking_no_author_attr(logical_line, tokens):
    """__author__ should not be used.

    S362: __author__ = slukjanov
    """
    for token_type, text, start_index, _, _ in tokens:
        if token_type == tokenize.NAME and text == "__author__":
            yield (start_index[1],
                   "S362: __author__ should not be used")


def check_oslo_namespace_imports(logical_line):
    """Check to prevent old oslo namespace usage.

    S363
    """
    if re.match(RE_OSLO_IMPORTS[0], logical_line):
        yield(0, "S363: '%s' must be used instead of '%s'." % (
            logical_line.replace('oslo.', 'oslo_'),
            logical_line))

    if re.match(RE_OSLO_IMPORTS[1], logical_line):
        yield(0, "S363: '%s' must be used instead of '%s'" % (
              'import oslo_%s' % logical_line.split()[-1],
              logical_line))


def dict_constructor_with_list_copy(logical_line):
    """Check to prevent dict constructor with a sequence of key-value pairs.

    S368
    """
    if RE_DICT_CONSTRUCTOR_WITH_LIST_COPY.match(logical_line):
        yield (0, 'S368: Must use a dict comprehension instead of a dict '
                  'constructor with a sequence of key-value pairs.')


def use_jsonutils(logical_line, filename):
    """Check to prevent importing json in sahara code.

    S375
    """
    if pycodestyle.noqa(logical_line):
        return
    if (RE_USE_JSONUTILS_INVALID_LINE.match(logical_line) and
            not RE_USE_JSONUTILS_VALID_LINE.match(logical_line)):
        yield(0, "S375: Use jsonutils from oslo_serialization instead"
                 " of json")


def no_mutable_default_args(logical_line):
    """Check to prevent mutable default argument in sahara code.

    S360
    """
    msg = "S360: Method's default argument shouldn't be mutable!"
    if RE_MUTABLE_DEFAULT_ARGS.match(logical_line):
        yield (0, msg)


def factory(register):
    register(import_db_only_in_conductor)
    register(hacking_no_author_attr)
    register(check_oslo_namespace_imports)
    register(commit_message.OnceGitCheckCommitTitleBug)
    register(commit_message.OnceGitCheckCommitTitleLength)
    register(import_checks.hacking_import_groups)
    register(import_checks.hacking_import_groups_together)
    register(dict_constructor_with_list_copy)
    register(logging_checks.no_translate_logs)
    register(logging_checks.accepted_log_levels)
    register(use_jsonutils)
    register(no_mutable_default_args)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from sahara.conductor import manager
from sahara import context
import sahara.tests.unit.conductor.base as test_base
from sahara.tests.unit.conductor.manager import test_clusters
from sahara.utils import general


class DefaultsTest(test_base.ConductorManagerTestCase):
    def __init__(self, *args, **kwargs):
        super(DefaultsTest, self).__init__(
            checks=[
                lambda: test_clusters.SAMPLE_CLUSTER,
                lambda: manager.CLUSTER_DEFAULTS,
                lambda: manager.NODE_GROUP_DEFAULTS,
                lambda: manager.INSTANCE_DEFAULTS,
            ], *args, **kwargs)

    def _assert_props(self, obj, **rules):
        for k, v in six.iteritems(rules):
            self.assertIn(k, obj)
            self.assertEqual(v, obj[k])

    def test_apply_defaults(self):
        self.assertEqual(
            {"a": 1, "b": 2, "c": 0},
            manager._apply_defaults({"c": 0, "b": 2},
                                    {"a": 1, "b": 2, "c": 3}))

    def _create_sample_cluster(self):
        ctx = context.ctx()
        cluster = self.api.cluster_create(ctx, test_clusters.SAMPLE_CLUSTER)
        self.assertIsInstance(cluster, dict)
        return cluster

    def test_cluster_defaults(self):
        cluster = self._create_sample_cluster()

        self._assert_props(cluster,
                           status="undefined",
                           status_description="",
                           info={})

    def test_node_group_defaults(self):
        cluster = self._create_sample_cluster()

        for ng in cluster['node_groups']:
            self._assert_props(ng,
                               node_configs={},
                               volumes_per_node=0,
                               volumes_size=0,
                               volume_mount_prefix="/volumes/disk")

    def test_instance_defaults(self):
        ctx = context.ctx()
        cluster = self._create_sample_cluster()
        cluster_id = cluster["id"]
        ng_id = cluster["node_groups"][-1]["id"]

        self.api.instance_add(ctx, ng_id, {
            "instance_name": "vm123"
        })

        cluster = self.api.cluster_get(ctx, cluster_id)
        ng = general.find_dict(cluster['node_groups'], id=ng_id)
        instance = general.find_dict(ng['instances'], instance_name="vm123")

        self._assert_props(instance, volumes=[])



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

import mock
from oslo_utils import uuidutils
import six
from sqlalchemy import exc as sa_ex
import testtools

from sahara.conductor import manager
from sahara import context
from sahara.db.sqlalchemy import models as m
from sahara import exceptions as ex
from sahara.service.validations import cluster_template_schema as cl_schema
from sahara.service.validations import node_group_template_schema as ngt_schema
import sahara.tests.unit.conductor.base as test_base
import sahara.tests.unit.conductor.manager.test_clusters as cluster_tests


SAMPLE_NGT = {
    "name": "ngt_test",
    "flavor_id": "42",
    "plugin_name": "test_plugin",
    "hadoop_version": "test_version",
    "node_processes": ["p1", "p2"],
    "image_id": uuidutils.generate_uuid(),
    "node_configs": {
        "service_1": {
            "config_1": "value_1"
        },
        "service_2": {
            "config_1": "value_1"
        }
    },
    "volumes_per_node": 1,
    "volumes_size": 1,
    "volume_type": "big",
    "volumes_availability_zone": "here",
    "volume_mount_prefix": "/tmp",
    "description": "my template",
    "floating_ip_pool": "public",
    "security_groups": ["cat", "dog"],
    "auto_security_group": False,
    "availability_zone": "here",
    "is_proxy_gateway": False,
    "volume_local_to_instance": False,
    'use_autoconfig': True,
    "is_public": False,
    "is_protected": False
}

SAMPLE_CLT = {
    "name": "clt_test",
    "plugin_name": "test_plugin",
    "hadoop_version": "test_version",
    "default_image_id": uuidutils.generate_uuid(),
    "cluster_configs": {
        "service_1": {
            "config_1": "value_1"
        },
        "service_2": {
            "config_1": "value_1"
        }
    },
    "node_groups": [
        {
            "name": "ng_1",
            "flavor_id": "42",
            "node_processes": ["p1", "p2"],
            "count": 1,
            "floating_ip_pool": None,
            "security_groups": None,
            "availability_zone": None,
            'use_autoconfig': True,
            "shares": None
        },
        {
            "name": "ng_2",
            "flavor_id": "42",
            "node_processes": ["p3", "p4"],
            "count": 3,
            "floating_ip_pool": None,
            "security_groups": ["group1", "group2"],
            "availability_zone": None,
            'use_autoconfig': True,
            "shares": None
        }

    ],
    "anti_affinity": ["datanode"],
    "description": "my template",
    "neutron_management_network": uuidutils.generate_uuid(),
    "shares": None,
    "is_public": False,
    "is_protected": False
}


class NodeGroupTemplates(test_base.ConductorManagerTestCase):
    def __init__(self, *args, **kwargs):
        super(NodeGroupTemplates, self).__init__(
            checks=[
                lambda: SAMPLE_CLT,
                lambda: SAMPLE_NGT,
                lambda: manager.CLUSTER_DEFAULTS,
                lambda: manager.NODE_GROUP_DEFAULTS,
                lambda: manager.INSTANCE_DEFAULTS,
            ], *args, **kwargs)

    def test_minimal_ngt_create_list_delete(self):
        ctx = context.ctx()
        self.api.node_group_template_create(ctx, SAMPLE_NGT)

        lst = self.api.node_group_template_get_all(ctx)
        self.assertEqual(1, len(lst))

        ngt_id = lst[0]['id']
        self.api.node_group_template_destroy(ctx, ngt_id)

        lst = self.api.node_group_template_get_all(ctx)
        self.assertEqual(0, len(lst))

    def test_duplicate_ngt_create(self):
        ctx = context.ctx()
        self.api.node_group_template_create(ctx, SAMPLE_NGT)
        with testtools.ExpectedException(ex.DBDuplicateEntry):
            self.api.node_group_template_create(ctx, SAMPLE_NGT)

    def test_ngt_fields(self):
        ctx = context.ctx()
        ngt_db_obj_id = self.api.node_group_template_create(
            ctx, SAMPLE_NGT)['id']

        ngt_db_obj = self.api.node_group_template_get(ctx, ngt_db_obj_id)
        self.assertIsInstance(ngt_db_obj, dict)

        for key, val in SAMPLE_NGT.items():
            self.assertEqual(val, ngt_db_obj.get(key),
                             "Key not found %s" % key)

    def test_ngt_delete(self):
        ctx = context.ctx()
        db_obj_ngt = self.api.node_group_template_create(ctx, SAMPLE_NGT)
        _id = db_obj_ngt['id']

        self.api.node_group_template_destroy(ctx, _id)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.node_group_template_destroy(ctx, _id)

    def test_ngt_delete_default(self):
        ctx = context.ctx()
        vals = copy.copy(SAMPLE_NGT)
        vals["name"] = "protected"
        vals["is_protected"] = True
        ngt_prot = self.api.node_group_template_create(ctx, vals)
        ngt_prot_id = ngt_prot['id']

        vals["name"] = "protected_default"
        vals["is_protected"] = True
        vals["is_default"] = True
        ngt_prot_def = self.api.node_group_template_create(ctx, vals)
        ngt_prot_def_id = ngt_prot_def['id']

        # We should not be able to delete ngt_prot until we remove
        # the protected flag, even if we pass ignore_prot_on_def
        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.node_group_template_destroy(ctx, ngt_prot_id)

        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.node_group_template_destroy(ctx, ngt_prot_id,
                                                 ignore_prot_on_def=True)

        update_values = {"is_protected": False}
        self.api.node_group_template_update(ctx,
                                            ngt_prot_id,
                                            update_values)
        self.api.node_group_template_destroy(ctx, ngt_prot_id)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.node_group_template_destroy(ctx, ngt_prot_id)

        # However, for the protected_default we should be able to
        # override the protected check by passing ignore_prot_on_def
        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.node_group_template_destroy(ctx, ngt_prot_def_id)

        self.api.node_group_template_destroy(ctx, ngt_prot_def_id,
                                             ignore_prot_on_def=True)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.node_group_template_destroy(ctx, ngt_prot_def_id)

    def test_ngt_search(self):
        ctx = context.ctx()
        ngt = copy.deepcopy(SAMPLE_NGT)
        ngt["name"] = "frederica"
        ngt["plugin_name"] = "test plugin"

        self.api.node_group_template_create(ctx, ngt)
        lst = self.api.node_group_template_get_all(ctx)
        self.assertEqual(1, len(lst))

        # Exact match
        kwargs = {'name': ngt['name'],
                  'plugin_name': ngt['plugin_name']}
        lst = self.api.node_group_template_get_all(ctx, **kwargs)
        self.assertEqual(1, len(lst))

        # Valid field but no matching value
        kwargs = {'name': ngt['name']+"foo"}
        lst = self.api.node_group_template_get_all(ctx, **kwargs)
        self.assertEqual(0, len(lst))

        # Valid field with substrings
        kwargs = {'name': "red",
                  'plugin_name': "test"}
        lst = self.api.node_group_template_get_all(ctx, **kwargs)
        self.assertEqual(0, len(lst))

        # Invalid field
        self.assertRaises(sa_ex.InvalidRequestError,
                          self.api.node_group_template_get_all,
                          ctx, **{'badfield': 'junk'})

    @mock.patch('sahara.db.sqlalchemy.api.regex_filter')
    def test_ngt_search_regex(self, regex_filter):

        # do this so we can return the correct value
        def _regex_filter(query, cls, regex_cols, search_opts):
            return query, search_opts

        regex_filter.side_effect = _regex_filter

        ctx = context.ctx()
        self.api.node_group_template_get_all(ctx)
        self.assertEqual(0, regex_filter.call_count)

        self.api.node_group_template_get_all(ctx,
                                             regex_search=True, name="fox")
        self.assertEqual(1, regex_filter.call_count)
        args, kwargs = regex_filter.call_args
        self.assertIs(args[1], m.NodeGroupTemplate)
        self.assertEqual(args[2], ["name", "description", "plugin_name",
                                   "tenant_id"])
        self.assertEqual(args[3], {"name": "fox"})

    def test_ngt_update(self):
        ctx = context.ctx()
        ngt = self.api.node_group_template_create(ctx, SAMPLE_NGT)
        ngt_id = ngt["id"]

        UPDATE_NAME = "UpdatedSampleNGTName"
        update_values = {"name": UPDATE_NAME}
        updated_ngt = self.api.node_group_template_update(ctx,
                                                          ngt_id,
                                                          update_values)
        self.assertEqual(UPDATE_NAME, updated_ngt["name"])

        updated_ngt = self.api.node_group_template_get(ctx, ngt_id)
        self.assertEqual(UPDATE_NAME, updated_ngt["name"])

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.node_group_template_update(ctx, -1, update_values)

        ngt = self.api.node_group_template_create(ctx, SAMPLE_NGT)
        ngt_id = ngt['id']
        with testtools.ExpectedException(ex.DBDuplicateEntry):
            self.api.node_group_template_update(ctx, ngt_id, update_values)

    def test_ngt_update_default(self):
        ctx = context.ctx()
        vals = copy.copy(SAMPLE_NGT)
        vals["name"] = "protected"
        vals["is_protected"] = True
        ngt_prot = self.api.node_group_template_create(ctx, vals)
        ngt_prot_id = ngt_prot["id"]

        vals["name"] = "protected_default"
        vals["is_protected"] = True
        vals["is_default"] = True
        ngt_prot_def = self.api.node_group_template_create(ctx, vals)
        ngt_prot_def_id = ngt_prot_def["id"]

        # We should not be able to update ngt_prot until we remove
        # the is_protected flag, even if we pass ignore_prot_on_def
        UPDATE_NAME = "UpdatedSampleNGTName"
        update_values = {"name": UPDATE_NAME}
        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.node_group_template_update(ctx,
                                                ngt_prot_id,
                                                update_values)

        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.node_group_template_update(ctx,
                                                ngt_prot_id,
                                                update_values,
                                                ignore_prot_on_def=True)
        update_values["is_protected"] = False
        updated_ngt = self.api.node_group_template_update(ctx,
                                                          ngt_prot_id,
                                                          update_values)
        self.assertEqual(UPDATE_NAME, updated_ngt["name"])

        # However, for the ngt_prot_def we should be able to
        # override the is_protected check by passing ignore_prot_on_def
        update_values = {"name": UPDATE_NAME+"default"}
        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.node_group_template_update(ctx,
                                                ngt_prot_def_id,
                                                update_values)

        updated_ngt = self.api.node_group_template_update(
            ctx, ngt_prot_def_id, update_values, ignore_prot_on_def=True)

        self.assertEqual(UPDATE_NAME+"default", updated_ngt["name"])
        self.assertTrue(updated_ngt["is_protected"])
        self.assertTrue(updated_ngt["is_default"])

    def test_ngt_update_with_nulls(self):
        ctx = context.ctx()
        ngt = self.api.node_group_template_create(ctx, SAMPLE_NGT)
        ngt_id = ngt["id"]

        updated_values = copy.deepcopy(SAMPLE_NGT)
        for prop, value in six.iteritems(
                ngt_schema.NODE_GROUP_TEMPLATE_SCHEMA["properties"]):
            if type(value["type"]) is list and "null" in value["type"]:
                updated_values[prop] = None

        # Prove that we can call update on these fields with null values
        # without an exception
        self.api.node_group_template_update(ctx,
                                            ngt_id,
                                            updated_values)

        updated_ngt = self.api.node_group_template_get(ctx, ngt_id)
        for prop, value in six.iteritems(updated_values):
            if value is None:
                self.assertIsNone(updated_ngt[prop])

    def test_ngt_update_delete_when_protected(self):
        ctx = context.ctx()
        sample = copy.deepcopy(SAMPLE_NGT)
        sample['is_protected'] = True
        ngt = self.api.node_group_template_create(ctx, sample)
        ngt_id = ngt["id"]

        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.node_group_template_update(ctx, ngt_id,
                                                {"name": "tmpl"})

        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.node_group_template_destroy(ctx, ngt_id)

        self.api.node_group_template_update(ctx, ngt_id,
                                            {"name": "tmpl",
                                             "is_protected": False})

    def test_public_ngt_update_from_another_tenant(self):
        ctx = context.ctx()
        sample = copy.deepcopy(SAMPLE_NGT)
        sample['is_public'] = True
        ngt = self.api.node_group_template_create(ctx, sample)
        ngt_id = ngt["id"]
        ctx.tenant_id = 'tenant_2'

        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.node_group_template_update(ctx, ngt_id,
                                                {"name": "tmpl"})

    def test_public_ngt_delete_from_another_tenant(self):
        ctx = context.ctx()
        sample = copy.deepcopy(SAMPLE_NGT)
        sample['is_public'] = True
        ngt = self.api.node_group_template_create(ctx, sample)
        ngt_id = ngt["id"]
        ctx.tenant_id = 'tenant_2'

        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.node_group_template_destroy(ctx, ngt_id)


class ClusterTemplates(test_base.ConductorManagerTestCase):
    def __init__(self, *args, **kwargs):
        super(ClusterTemplates, self).__init__(
            checks=[
                lambda: SAMPLE_CLT,
                lambda: SAMPLE_NGT,
                lambda: manager.CLUSTER_DEFAULTS,
                lambda: manager.NODE_GROUP_DEFAULTS,
                lambda: manager.INSTANCE_DEFAULTS,
            ], *args, **kwargs)

    def test_minimal_clt_create_list_delete(self):
        ctx = context.ctx()
        self.api.cluster_template_create(ctx, SAMPLE_CLT)

        lst = self.api.cluster_template_get_all(ctx)
        self.assertEqual(1, len(lst))

        clt_id = lst[0]['id']
        self.api.cluster_template_destroy(ctx, clt_id)

        lst = self.api.cluster_template_get_all(ctx)
        self.assertEqual(0, len(lst))

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.cluster_template_destroy(ctx, clt_id)

    def test_duplicate_clt_create(self):
        ctx = context.ctx()
        self.api.cluster_template_create(ctx, SAMPLE_CLT)
        with testtools.ExpectedException(ex.DBDuplicateEntry):
            self.api.cluster_template_create(ctx, SAMPLE_CLT)

    def test_clt_fields(self):
        ctx = context.ctx()
        clt_db_obj_id = self.api.cluster_template_create(ctx, SAMPLE_CLT)['id']

        clt_db_obj = self.api.cluster_template_get(ctx, clt_db_obj_id)
        self.assertIsInstance(clt_db_obj, dict)

        for key, val in SAMPLE_CLT.items():
            if key == 'node_groups':
                # this will be checked separately
                continue
            self.assertEqual(val, clt_db_obj.get(key),
                             "Key not found %s" % key)

        for ng in clt_db_obj["node_groups"]:
            ng.pop("created_at")
            ng.pop("updated_at")
            ng.pop("id")
            ng.pop("tenant_id")
            self.assertEqual(clt_db_obj_id, ng.pop("cluster_template_id"))
            ng.pop("image_id")
            ng.pop("node_configs")
            ng.pop("node_group_template_id")
            ng.pop("volume_mount_prefix")
            ng.pop("volumes_size")
            ng.pop("volumes_per_node")
            ng.pop("volumes_availability_zone")
            ng.pop("volume_type")
            ng.pop("auto_security_group")
            ng.pop("is_proxy_gateway")
            ng.pop("boot_from_volume")
            ng.pop("boot_volume_type")
            ng.pop("boot_volume_availability_zone")
            ng.pop("boot_volume_local_to_instance")
            ng.pop('volume_local_to_instance')

        self.assertEqual(SAMPLE_CLT["node_groups"],
                         clt_db_obj["node_groups"])

    def test_clt_delete(self):
        ctx = context.ctx()
        db_obj_clt = self.api.cluster_template_create(ctx, SAMPLE_CLT)
        _id = db_obj_clt['id']

        self.api.cluster_template_destroy(ctx, _id)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.cluster_template_destroy(ctx, _id)

    def test_clt_delete_default(self):
        ctx = context.ctx()
        vals = copy.copy(SAMPLE_CLT)
        vals["name"] = "protected"
        vals["is_protected"] = True
        clt_prot = self.api.cluster_template_create(ctx, vals)
        clt_prot_id = clt_prot['id']

        vals["name"] = "protected_default"
        vals["is_protected"] = True
        vals["is_default"] = True
        clt_prot_def = self.api.cluster_template_create(ctx, vals)
        clt_prot_def_id = clt_prot_def['id']

        # We should not be able to delete clt_prot until we remove
        # the is_protected flag, even if we pass ignore_prot_on_def
        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.cluster_template_destroy(ctx, clt_prot_id)

        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.cluster_template_destroy(ctx, clt_prot_id,
                                              ignore_prot_on_def=True)

        update_values = {"is_protected": False}
        self.api.cluster_template_update(ctx,
                                         clt_prot_id,
                                         update_values)
        self.api.cluster_template_destroy(ctx, clt_prot_id)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.cluster_template_destroy(ctx, clt_prot_id)

        # However, for clt_prot_def we should be able to override
        # the is_protected check by passing ignore_prot_on_def
        with testtools.ExpectedException(ex.DeletionFailed):
            self.api.cluster_template_destroy(ctx, clt_prot_def_id)

        self.api.cluster_template_destroy(ctx, clt_prot_def_id,
                                          ignore_prot_on_def=True)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.cluster_template_destroy(ctx, clt_prot_def_id)

    def test_clt_search(self):
        ctx = context.ctx()
        clt = copy.deepcopy(SAMPLE_CLT)
        clt["name"] = "frederica"
        clt["plugin_name"] = "test_plugin"

        self.api.cluster_template_create(ctx, clt)
        lst = self.api.cluster_template_get_all(ctx)
        self.assertEqual(1, len(lst))

        # Exact match
        kwargs = {'name': clt['name'],
                  'plugin_name': clt['plugin_name']}
        lst = self.api.cluster_template_get_all(ctx, **kwargs)
        self.assertEqual(1, len(lst))

        # Valid field but no matching value
        kwargs = {'name': clt['name']+"foo"}
        lst = self.api.cluster_template_get_all(ctx, **kwargs)
        self.assertEqual(0, len(lst))

        # Valid field with substrings
        kwargs = {'name': "red",
                  'plugin_name': "test"}
        lst = self.api.cluster_template_get_all(ctx, **kwargs)
        self.assertEqual(0, len(lst))

        # Invalid field
        self.assertRaises(sa_ex.InvalidRequestError,
                          self.api.cluster_template_get_all,
                          ctx, **{'badfield': 'junk'})

    @mock.patch('sahara.db.sqlalchemy.api.regex_filter')
    def test_clt_search_regex(self, regex_filter):

        # do this so we can return the correct value
        def _regex_filter(query, cls, regex_cols, search_opts):
            return query, search_opts

        regex_filter.side_effect = _regex_filter

        ctx = context.ctx()
        self.api.cluster_template_get_all(ctx)
        self.assertEqual(0, regex_filter.call_count)

        self.api.cluster_template_get_all(ctx, regex_search=True, name="fox")
        self.assertEqual(1, regex_filter.call_count)
        args, kwargs = regex_filter.call_args
        self.assertIs(args[1], m.ClusterTemplate)
        self.assertEqual(args[2], ["name", "description", "plugin_name",
                                   "tenant_id"])
        self.assertEqual(args[3], {"name": "fox"})

    def test_clt_update(self):
        ctx = context.ctx()
        clt = self.api.cluster_template_create(ctx, SAMPLE_CLT)
        clt_id = clt["id"]

        UPDATE_NAME = "UpdatedClusterTemplate"
        update_values = {"name": UPDATE_NAME}
        updated_clt = self.api.cluster_template_update(ctx,
                                                       clt_id,
                                                       update_values)
        self.assertEqual(UPDATE_NAME, updated_clt["name"])

        updated_clt = self.api.cluster_template_get(ctx, clt_id)
        self.assertEqual(UPDATE_NAME, updated_clt["name"])
        self.assertEqual(clt["node_groups"], updated_clt["node_groups"])

        # check duplicate name handling
        clt = self.api.cluster_template_create(ctx, SAMPLE_CLT)
        clt_id = clt["id"]
        with testtools.ExpectedException(ex.DBDuplicateEntry):
            self.api.cluster_template_update(ctx, clt_id, update_values)

        with testtools.ExpectedException(ex.NotFoundException):
            self.api.cluster_template_update(ctx, -1, update_values)

        # create a cluster and try updating the referenced cluster template
        cluster_val = copy.deepcopy(cluster_tests.SAMPLE_CLUSTER)
        cluster_val['name'] = "ClusterTemplateUpdateTestCluster"
        cluster_val['cluster_template_id'] = clt['id']
        self.api.cluster_create(ctx, cluster_val)
        update_values = {"name": "noUpdateInUseName"}

        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.cluster_template_update(ctx, clt['id'], update_values)

    def test_clt_update_default(self):
        ctx = context.ctx()
        vals = copy.copy(SAMPLE_CLT)
        vals["name"] = "protected"
        vals["is_protected"] = True
        clt_prot = self.api.cluster_template_create(ctx, vals)
        clt_prot_id = clt_prot["id"]

        vals["name"] = "protected_default"
        vals["is_protected"] = True
        vals["is_default"] = True
        clt_prot_def = self.api.cluster_template_create(ctx, vals)
        clt_prot_def_id = clt_prot_def["id"]

        # We should not be able to update clt_prot until we remove
        # the is_protected flag, even if we pass ignore_prot_on_def
        UPDATE_NAME = "UpdatedClusterTemplate"
        update_values = {"name": UPDATE_NAME}
        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.cluster_template_update(ctx,
                                             clt_prot_id,
                                             update_values)

        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.cluster_template_update(ctx,
                                             clt_prot_id,
                                             update_values,
                                             ignore_prot_on_def=True)
        update_values["is_protected"] = False
        updated_clt = self.api.cluster_template_update(ctx,
                                                       clt_prot_id,
                                                       update_values)
        self.assertEqual(UPDATE_NAME, updated_clt["name"])

        # However, for the clt_prot_def we should be able to
        # override the is_protected check by passing ignore_prot_on_def
        update_values = {"name": UPDATE_NAME+"default"}
        with testtools.ExpectedException(ex.UpdateFailedException):
            self.api.cluster_template_update(ctx,
                                             clt_prot_def_id,
                                             update_values)

        updated_clt = self.api.cluster_template_update(ctx,
                                                       clt_prot_def_id,
                                                       update_values,
                                                       ignore_prot_on_def=True)
        self.assertEqual(UPDATE_NAME+"default", updated_clt["name"])
        self.assertTrue(updated_clt["is_default"])
        self.assertTrue(updated_clt["is_protected"])

    def test_clt_update_with_nulls(self):
        ctx = context.ctx()
        clt = self.api.cluster_template_create(ctx, SAMPLE_CLT)
        clt_id = clt["id"]

        updated_values = copy.deepcopy(SAMPLE_CLT)
        for prop, value in six.iteritems(
                cl_schema.CLUSTER_TEMPLATE_SCHEMA["properties"]):
            if type(value["type"]) is list and "null" in value["type"]:
                updated_values[prop] = None

        # Prove that we can call update on these fields with null values
        # without an exception
        self.api.cluster_template_update(ctx,
                                         clt_id,
                                         updated_values)

        updated_clt = self.api.cluster_template_get(ctx, clt_id)
        for prop, value in six.iteritems(updated_values):
            if value is None:
                # Conductor populates node groups with [] when
                # the value given is null
                if prop == "node_groups":
                    self.assertEqual([], updated_clt[prop])
                else:
                    self.assertIsNone(updated_clt[prop])

    def test_clt_update_delete_when_protected(self):
        ctx = context.ctx()
        sample = copy.deepcopy(SAMPLE_CLT)
        sample['is_protected'] = True
        clt = self.api.cluster_template_create(ctx, sample)
        clt_id = clt["id"]

        with testtools.ExpectedException(ex.UpdateFailedException):
            try:
                self.api.cluster_template_update(ctx, clt_id, {"name": "tmpl"})
            except ex.UpdateFailedException as e:
                self.assert_protected_resource_exception(e)
                raise

        with testtools.ExpectedException(ex.DeletionFailed):
            try:
                self.api.cluster_template_destroy(ctx, clt_id)
            except ex.DeletionFailed as e:
                self.assert_protected_resource_exception(e)
                raise

        self.api.cluster_template_update(ctx, clt_id,
                                         {"name": "tmpl",
                                          "is_protected": False})

    def test_public_clt_update_delete_from_another_tenant(self):
        ctx = context.ctx()
        sample = copy.deepcopy(SAMPLE_CLT)
        sample['is_public'] = True
        clt = self.api.cluster_template_create(ctx, sample)
        clt_id = clt["id"]
        ctx.tenant_id = 'tenant_2'

        with testtools.ExpectedException(ex.UpdateFailedException):
            try:
                self.api.cluster_template_update(ctx, clt_id,
                                                 {"name": "tmpl"})
            except ex.UpdateFailedException as e:
                self.assert_created_in_another_tenant_exception(e)
                raise

        with testtools.ExpectedException(ex.DeletionFailed):
            try:
                self.api.cluster_template_destroy(ctx, clt_id)
            except ex.DeletionFailed as e:
                self.assert_created_in_another_tenant_exception(e)
                raise

    def test_update_clt_on_ngt_update(self):
        # Prove that cluster templates get updated with proper values
        # after a referenced node group template is updated
        ctx = context.ctx()
        ngt = self.api.node_group_template_create(ctx, SAMPLE_NGT)
        sample = copy.deepcopy(SAMPLE_CLT)
        sample["node_groups"] = [
            {"node_group_template_id": ngt['id'],
             "count": 1}
        ]
        ct = self.api.cluster_template_create(ctx, sample)
        UPDATE_FLAVOR = "41"
        update_values = {"flavor_id": UPDATE_FLAVOR}
        self.api.node_group_template_update(ctx, ngt["id"], update_values)
        updated_ct = self.api.cluster_template_get(ctx, ct["id"])
        self.assertEqual(UPDATE_FLAVOR,
                         updated_ct["node_groups"][0]["flavor_id"])



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import six.moves.urllib.parse as urlparse

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp.data_sources.base import DataSourceType
from sahara.swift import swift_helper as sw
from sahara.swift import utils as su
from sahara.utils.types import FrozenDict

CONF = cfg.CONF


class SwiftType(DataSourceType):
    def validate(self, data):
        self._validate_url(data['url'])

        if not CONF.use_domain_for_proxy_users and "credentials" not in data:
            raise ex.InvalidCredentials(_("No credentials provided for Swift"))
        if not CONF.use_domain_for_proxy_users and (
                "user" not in data["credentials"]):
            raise ex.InvalidCredentials(
                _("User is not provided in credentials for Swift"))
        if not CONF.use_domain_for_proxy_users and (
                "password" not in data["credentials"]):
            raise ex.InvalidCredentials(
                _("Password is not provided in credentials for Swift"))

    def _validate_url(self, url):
        if len(url) == 0:
            raise ex.InvalidDataException(_("Swift url must not be empty"))

        url = urlparse.urlparse(url)
        if url.scheme != "swift":
            raise ex.InvalidDataException(_("URL scheme must be 'swift'"))

        # The swift url suffix does not have to be included in the netloc.
        # However, if the swift suffix indicator is part of the netloc then
        # we require the right suffix.
        # Additionally, the path must be more than '/'
        if (su.SWIFT_URL_SUFFIX_START in url.netloc and not
           url.netloc.endswith(su.SWIFT_URL_SUFFIX)) or len(url.path) <= 1:
            raise ex.InvalidDataException(
                _("URL must be of the form swift://container%s/object")
                % su.SWIFT_URL_SUFFIX)

    def prepare_cluster(self, data_source, cluster, **kwargs):
        if hasattr(data_source, "credentials"):
            job_configs = kwargs.pop('job_configs')

            # if no data source was passed as a reference for the job, the
            # job_configs will not be changed (so it will be a FronzenDict)
            # and we don't need to change it as well
            if isinstance(job_configs, FrozenDict) or \
               job_configs.get('configs', None) is None:
                return

            if not job_configs.get('proxy_configs'):
                username = data_source.credentials['user']
                password = data_source.credentials['password']

                # Don't overwrite if there is already a value here
                if job_configs['configs'].get(sw.HADOOP_SWIFT_USERNAME, None) \
                   is None and (username is not None):
                    job_configs['configs'][sw.HADOOP_SWIFT_USERNAME] = username
                if job_configs['configs'].get(sw.HADOOP_SWIFT_PASSWORD, None) \
                   is None and (password is not None):
                    job_configs['configs'][sw.HADOOP_SWIFT_PASSWORD] = password



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

from oslo_log import log as logging
import six

from sahara import conductor as cond
from sahara import context
from sahara.utils.openstack import nova

conductor = cond.API

LOG = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class AutoConfigsProvider(object):
    def __init__(self, mapper, plugin_configs, cluster, scaling):
        """This meta class provides general recommendation utils for cluster

        configuration.
        :param mapper: dictionary, that describes which cluster configs and
        node_configs to configure. It should maps to following dicts:
        node_configs to configure and cluster_configs to configure. This
        dicts should contains abstract names of configs as keys and
        tuple (correct_applicable_target, correct_name) as values. Such
        representation allows to use same AutoConfigsProvider for plugins
        with almost same configs and configuring principles.
        :param plugin_configs: all plugins_configs for specified plugin
        :param cluster: cluster which is required to configure
        :param scaling: indicates that current cluster operation is scaling
        """
        self.plugin_configs = plugin_configs
        self.cluster = cluster
        self.node_configs_to_update = mapper.get('node_configs', {})
        self.cluster_configs_to_update = mapper.get('cluster_configs', {})
        self.scaling = scaling

    @abc.abstractmethod
    def _get_recommended_node_configs(self, node_group):
        """Method calculates and returns recommended configs for node_group.

        It's not required to update node_configs of node_group using the
        conductor api in this method, because it will be done in the method
        apply_node_configs.

        :param node_group: NodeGroup Sahara resource.
        :return: dictionary with calculated recommended configs for
        node_group.
        """
        pass

    @abc.abstractmethod
    def _get_recommended_cluster_configs(self):
        """Method calculates and returns recommended configs for cluster.

        It's not required to update cluster_configs of cluster using the
        conductor api in this method, because it will be done in the method
        apply_cluster_configs.

        :return: dictionary with calculated recommended configs for
        cluster.
        """
        pass

    def _can_be_recommended(self, configs_list, node_group=None):
        """Method calculates and returns True, when it's possible to

        automatically configure provided list of configs configs_list.
        Otherwise, method should return False.

        :param configs_list: list of configs which we want to configure
        :param node_group: optional argument, which should be provided if
        some config can be used in node_configs of some node_group
        :return: True if all configs can be configured and False otherwise
        """
        # cluster configs is Frozen Dict, so let's call to_dict()
        cl_configs = self.cluster.cluster_configs.to_dict()
        for ncfg in configs_list:
            section, name = self._get_correct_section_and_name(ncfg)
            if section in cl_configs and name in cl_configs[section]:
                return False

        if not node_group:
            return True

        cl_configs = node_group.node_configs.to_dict()
        for ncfg in configs_list:
            section, name = self._get_correct_section_and_name(ncfg)
            if section in cl_configs and name in cl_configs[section]:
                return False
        return True

    def _get_correct_section_and_name(self, config_name):
        """Calculates and returns correct applicable target and name from

        abstract name of config.
        :param config_name: abstract name of config.
        :return: correct applicable target and name for config_name
        """
        section, name = None, None
        if config_name in self.cluster_configs_to_update:
            section = self.cluster_configs_to_update[config_name][0]
            name = self.cluster_configs_to_update[config_name][1]
        elif config_name in self.node_configs_to_update:
            section = self.node_configs_to_update[config_name][0]
            name = self.node_configs_to_update[config_name][1]
        return section, name

    def _get_default_config_value(self, config_name):
        """Calculates and return default value of config from

        abstract name of config.
        :param config_name: abstract name of config.
        :return: default config value for config_name.
        """
        section, name = self._get_correct_section_and_name(config_name)
        for config in self.plugin_configs:
            if config.applicable_target == section and config.name == name:
                return config.default_value

    def _merge_configs(self, current_configs, proposed_configs):
        """Correctly merges old configs and new extra configs"""
        result = {}
        for (section, configs) in six.iteritems(proposed_configs):
            cfg_values = {}
            if section in current_configs:
                cfg_values = (current_configs[section] if
                              current_configs[section] else {})
            cfg_values.update(configs)
            result.update({section: cfg_values})
        for (section, configs) in six.iteritems(current_configs):
            if section not in result:
                result.update({section: configs})
        return result

    def _get_cluster_extra(self):
        cluster = self.cluster
        return cluster.extra.to_dict() if cluster.extra else {}

    def finalize_autoconfiguration(self):
        if not self.cluster.use_autoconfig:
            return
        cluster_extra = self._get_cluster_extra()
        cluster_extra['auto-configured'] = True
        conductor.cluster_update(
            context.ctx(), self.cluster, {'extra': cluster_extra})

    def apply_node_configs(self, node_group):
        """Method applies configs for node_group using conductor api,

        which were calculated with recommend_node_configs method.
        :param node_group: NodeGroup Sahara resource.
        :return: None.
        """
        if not node_group.use_autoconfig or not self.cluster.use_autoconfig:
            return
        to_update = self.node_configs_to_update
        recommended_node_configs = self._get_recommended_node_configs(
            node_group)
        if not recommended_node_configs:
            # Nothing to configure
            return
        current_dict = node_group.node_configs.to_dict()
        configuration = {}
        for ncfg in six.iterkeys(to_update):
            if ncfg not in recommended_node_configs:
                continue
            n_section = to_update[ncfg][0]
            n_name = to_update[ncfg][1]
            proposed_config_value = recommended_node_configs[ncfg]
            if n_section not in configuration:
                configuration.update({n_section: {}})
            configuration[n_section].update({n_name: proposed_config_value})
        current_dict = self._merge_configs(current_dict, configuration)
        conductor.node_group_update(context.ctx(), node_group,
                                    {'node_configs': current_dict})

    def apply_cluster_configs(self):
        """Method applies configs for cluster using conductor api, which were

        calculated with recommend_cluster_configs method.
        :return: None.
        """
        cluster = self.cluster
        if not cluster.use_autoconfig:
            return
        to_update = self.cluster_configs_to_update
        recommended_cluster_configs = self._get_recommended_cluster_configs()
        if not recommended_cluster_configs:
            # Nothing to configure
            return
        current_dict = cluster.cluster_configs.to_dict()
        configuration = {}
        for ncfg in six.iterkeys(to_update):
            if ncfg not in recommended_cluster_configs:
                continue
            n_section = to_update[ncfg][0]
            n_name = to_update[ncfg][1]
            proposed_config_value = recommended_cluster_configs[ncfg]
            if n_section not in configuration:
                configuration.update({n_section: {}})
            configuration[n_section].update({n_name: proposed_config_value})
        current_dict = self._merge_configs(current_dict, configuration)
        conductor.cluster_update(context.ctx(), cluster,
                                 {'cluster_configs': current_dict})

    def apply_recommended_configs(self):
        """Method applies recommended configs for cluster and for all

        node_groups using conductor api.
        :return: None.
        """
        if self.scaling:
            # Validate cluster is not an old created cluster
            cluster_extra = self._get_cluster_extra()
            if 'auto-configured' not in cluster_extra:
                # Don't configure
                return

        for ng in self.cluster.node_groups:
            self.apply_node_configs(ng)
        self.apply_cluster_configs()
        configs = list(self.cluster_configs_to_update.keys())
        configs.extend(list(self.node_configs_to_update.keys()))
        LOG.debug("Following configs were auto-configured: {configs}".format(
            configs=configs))
        self.finalize_autoconfiguration()


class HadoopAutoConfigsProvider(AutoConfigsProvider):
    def __init__(self, mapper, plugin_configs, cluster, scaling, hbase=False):
        super(HadoopAutoConfigsProvider, self).__init__(
            mapper, plugin_configs, cluster, scaling)
        self.requested_flavors = {}
        self.is_hbase_enabled = hbase

    def _get_java_opts(self, value):
        return "-Xmx%dm" % int(value)

    def _transform_mb_to_gb(self, mb):
        return mb / 1024.

    def _transform_gb_to_mb(self, gb):
        return gb * 1024.

    def _get_min_size_of_container(self, ram):
        if ram <= 4:
            return 256
        if ram <= 8:
            return 512
        if ram <= 24:
            return 1024
        return 2048

    def _get_os_ram_recommendation(self, ram):
        upper_bounds = [4, 8, 16, 24, 48, 64, 72, 96, 128, 256]
        reserve_for_os = [1, 2, 2, 4, 6, 8, 8, 12, 24, 32]
        for (upper, reserve) in zip(upper_bounds, reserve_for_os):
            if ram <= upper:
                return reserve
        return 64

    def _get_hbase_ram_recommendations(self, ram):
        if not self.is_hbase_enabled:
            return 0
        upper_bounds = [4, 8, 16, 24, 48, 64, 72, 96, 128, 256]
        reserve_for_hbase = [1, 1, 2, 4, 8, 8, 8, 16, 24, 32]
        for (upper, reserve) in zip(upper_bounds, reserve_for_hbase):
            if ram <= upper:
                return reserve
        return 64

    def _get_node_group_data(self, node_group):
        if node_group.flavor_id not in self.requested_flavors:
            flavor = nova.get_flavor(id=node_group.flavor_id)
            self.requested_flavors[node_group.flavor_id] = flavor
        else:
            flavor = self.requested_flavors[node_group.flavor_id]
        cpu = flavor.vcpus
        ram = flavor.ram
        data = {}
        # config recommendations was taken from Ambari code
        os = self._get_os_ram_recommendation(self._transform_mb_to_gb(ram))
        hbase = self._get_hbase_ram_recommendations(
            self._transform_mb_to_gb(ram))
        reserved_ram = self._transform_gb_to_mb(os + hbase)
        min_container_size = self._get_min_size_of_container(
            self._transform_mb_to_gb(ram))

        # we use large amount of containers to allow users to run
        # at least 4 jobs at same time on clusters based on small flavors
        data["containers"] = int(max(
            8, min(2 * cpu, ram / min_container_size)))
        data["ramPerContainer"] = (ram - reserved_ram) / data["containers"]
        data["ramPerContainer"] = max(data["ramPerContainer"],
                                      min_container_size)
        data["ramPerContainer"] = min(2048, int(data["ramPerContainer"]))

        data["ramPerContainer"] = int(data["ramPerContainer"])
        data["mapMemory"] = int(data["ramPerContainer"])
        data["reduceMemory"] = int(2 * data["ramPerContainer"])
        data["amMemory"] = int(min(data["mapMemory"], data["reduceMemory"]))

        return data

    def _get_recommended_node_configs(self, node_group):
        """Calculates recommended MapReduce and YARN configs for specified

        node_group.
        :param node_group: NodeGroup Sahara resource
        :return: dictionary with recommended MapReduce and YARN configs
        """
        configs_to_update = list(self.node_configs_to_update.keys())
        if not self._can_be_recommended(configs_to_update, node_group):
            return {}
        data = self._get_node_group_data(node_group)
        r = {}
        r['yarn.nodemanager.resource.memory-mb'] = (data['containers'] *
                                                    data['ramPerContainer'])
        r['yarn.scheduler.minimum-allocation-mb'] = data['ramPerContainer']
        r['yarn.scheduler.maximum-allocation-mb'] = (data['containers'] *
                                                     data['ramPerContainer'])
        r['yarn.nodemanager.vmem-check-enabled'] = "false"
        r['yarn.app.mapreduce.am.resource.mb'] = data['amMemory']
        r['yarn.app.mapreduce.am.command-opts'] = self._get_java_opts(
            0.8 * data['amMemory'])
        r['mapreduce.map.memory.mb'] = data['mapMemory']
        r['mapreduce.reduce.memory.mb'] = data['reduceMemory']
        r['mapreduce.map.java.opts'] = self._get_java_opts(
            0.8 * data['mapMemory'])
        r['mapreduce.reduce.java.opts'] = self._get_java_opts(
            0.8 * data['reduceMemory'])
        r['mapreduce.task.io.sort.mb'] = int(min(
            0.4 * data['mapMemory'], 1024))
        return r

    def get_datanode_name(self):
        return "datanode"

    def _get_recommended_cluster_configs(self):
        """Method recommends dfs_replication for cluster.

        :return: recommended value of dfs_replication.
        """
        if not self._can_be_recommended(['dfs.replication']):
            return {}
        datanode_count = 0
        datanode_proc_name = self.get_datanode_name()
        for ng in self.cluster.node_groups:
            if datanode_proc_name in ng.node_processes:
                datanode_count += ng.count
        replica = 'dfs.replication'
        recommended_value = self._get_default_config_value(replica)
        if recommended_value:
            return {replica: min(recommended_value, datanode_count)}
        else:
            return {}



# Copyright (c) 2013 Hortonworks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from neutronclient.common import exceptions as n_ex
from neutronclient.neutron import client as neutron_cli
from oslo_config import cfg
from oslo_log import log as logging

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service import sessions
from sahara.utils.openstack import base
from sahara.utils.openstack import keystone


opts = [
    cfg.BoolOpt('api_insecure',
                default=False,
                help='Allow to perform insecure SSL requests to neutron.'),
    cfg.StrOpt('ca_file',
               help='Location of ca certificates file to use for neutron '
                    'client requests.'),
    cfg.StrOpt("endpoint_type",
               default="internalURL",
               help="Endpoint type for neutron client requests")
]

neutron_group = cfg.OptGroup(name='neutron',
                             title='Neutron client options')

CONF = cfg.CONF
CONF.register_group(neutron_group)
CONF.register_opts(opts, group=neutron_group)

LOG = logging.getLogger(__name__)


def client(auth=None):
    if not auth:
        auth = keystone.auth()
    session = sessions.cache().get_session(sessions.SESSION_TYPE_NEUTRON)
    neutron = neutron_cli.Client('2.0', session=session, auth=auth,
                                 endpoint_type=CONF.neutron.endpoint_type,
                                 region_name=CONF.os_region_name)
    return neutron


class NeutronClient(object):
    neutron = None
    routers = {}

    def __init__(self, network, token, tenant_name, auth=None):
        if not auth:
            auth = keystone.token_auth(token=token, project_name=tenant_name)
        self.neutron = client(auth)
        self.network = network

    def get_router(self):
        matching_router = NeutronClient.routers.get(self.network, None)
        if matching_router:
            LOG.debug('Returning cached qrouter')
            return matching_router['id']

        routers = self.neutron.list_routers()['routers']
        for router in routers:
            device_id = router['id']
            ports = base.execute_with_retries(
                self.neutron.list_ports, device_id=device_id)['ports']
            port = next((port for port in ports
                         if port['network_id'] == self.network), None)
            if port:
                matching_router = router
                NeutronClient.routers[self.network] = matching_router
                break

        if not matching_router:
            raise ex.SystemError(_('Neutron router corresponding to network '
                                   '%s is not found') % self.network)

        return matching_router['id']


def get_private_network_cidrs(cluster):
    neutron_client = client()
    private_net = base.execute_with_retries(neutron_client.show_network,
                                            cluster.neutron_management_network)

    cidrs = []
    for subnet_id in private_net['network']['subnets']:
        subnet = base.execute_with_retries(
            neutron_client.show_subnet, subnet_id)
        cidrs.append(subnet['subnet']['cidr'])

    return cidrs


def get_network(id):
    try:
        return base.execute_with_retries(
            client().find_resource_by_id, 'network', id)
    except n_ex.NotFound:
        return None



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from oslo_utils import uuidutils
import testtools

import sahara.exceptions as ex
from sahara.service.edp.data_sources.manila.implementation import ManilaType
from sahara.tests.unit import base


class _FakeShare(object):
    def __init__(self, id, share_proto='NFS'):
        self.id = id
        self.share_proto = share_proto


class TestManilaType(base.SaharaTestCase):
    def setUp(self):
        super(TestManilaType, self).setUp()
        self.manila_type = ManilaType()

    @mock.patch('sahara.utils.openstack.manila.client')
    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.service.edp.utils.shares.mount_shares')
    def test_prepare_cluster(self, mount_shares, cluster_update,
                             f_manilaclient):

        cluster_shares = [
            {'id': 'the_share_id',
             'path': '/mnt/mymountpoint'}
        ]

        cluster = mock.Mock()
        cluster.shares = cluster_shares

        # This should return a default path, and should cause
        # a mount at the default location
        share = _FakeShare("missing_id")
        f_manilaclient.return_value = mock.Mock(shares=mock.Mock(
            get=mock.Mock(return_value=share)))

        url = 'manila://missing_id/the_path'
        self.manila_type._prepare_cluster(url, cluster)

        self.assertEqual(1, mount_shares.call_count)
        self.assertEqual(1, cluster_update.call_count)

    @mock.patch('sahara.service.edp.utils.shares.get_share_path')
    @mock.patch('sahara.utils.openstack.manila.client')
    @mock.patch('sahara.conductor.API.cluster_update')
    @mock.patch('sahara.service.edp.utils.shares.mount_shares')
    def test_get_runtime_url(self, mount_shares, cluster_update,
                             f_manilaclient, get_share_path):

        # first it finds the path, then it doesn't so it has to mount it
        # and only then it finds it
        get_share_path.side_effect = ['/mnt/mymountpoint/the_path', None,
                                      '/mnt/missing_id/the_path']

        cluster = mock.Mock()
        cluster.shares = []
        url = 'manila://the_share_id/the_path'

        res = self.manila_type.get_runtime_url(url, cluster)
        self.assertEqual('file:///mnt/mymountpoint/the_path', res)
        self.assertEqual(0, mount_shares.call_count)
        self.assertEqual(0, cluster_update.call_count)

        # This should return a default path, and should cause
        # a mount at the default location
        share = _FakeShare("missing_id")
        f_manilaclient.return_value = mock.Mock(shares=mock.Mock(
            get=mock.Mock(return_value=share)))

        url = 'manila://missing_id/the_path'
        res = self.manila_type.get_runtime_url(url, cluster)
        self.assertEqual('file:///mnt/missing_id/the_path', res)
        self.assertEqual(1, mount_shares.call_count)
        self.assertEqual(1, cluster_update.call_count)

    def test_manila_type_validation_wrong_schema(self):
        data = {
            "name": "test_data_data_source",
            "url": "man://%s" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("incorrect url schema for")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

    def test_manila_type_validation_empty_url(self):
        data = {
            "name": "test_data_data_source",
            "url": "",
            "type": "manila",
            "description": ("empty url")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

    def test_manila_type_validation_no_uuid(self):
        data = {
            "name": "test_data_data_source",
            "url": "manila://bob",
            "type": "manila",
            "description": ("netloc is not a uuid")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

    def test_manila_type_validation_no_path(self):
        data = {
            "name": "test_data_data_source",
            "url": "manila://%s" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("netloc is not a uuid")
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.manila_type.validate(data)

    def test_manila_type_validation_correct(self):
        data = {
            "name": "test_data_data_source",
            "url": "manila://%s/foo" % uuidutils.generate_uuid(),
            "type": "manila",
            "description": ("correct url")
        }
        self.manila_type.validate(data)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.api import v10 as api
import sahara.service.validations.base as b
from sahara.service.validations import shares


def check_node_group_template_create(data, **kwargs):
    plugin_version = 'hadoop_version'
    if data.get('plugin_version'):
        plugin_version = 'plugin_version'

    b.check_node_group_template_unique_name(data['name'])
    b.check_plugin_name_exists(data['plugin_name'])
    b.check_plugin_supports_version(data['plugin_name'],
                                    data[plugin_version])
    b.check_node_group_basic_fields(data['plugin_name'],
                                    data[plugin_version], data)
    if data.get('image_id'):
        b.check_image_registered(data['image_id'])
        b.check_required_image_tags(data['plugin_name'],
                                    data[plugin_version],
                                    data['image_id'])
    if data.get('shares'):
        shares.check_shares(data['shares'])


def check_node_group_template_usage(node_group_template_id, **kwargs):
    cluster_users = []
    template_users = []

    for cluster in api.get_clusters():
        if (node_group_template_id in
            [node_group.node_group_template_id
             for node_group in cluster.node_groups]):
            cluster_users += [cluster.name]

    for cluster_template in api.get_cluster_templates():
        if (node_group_template_id in
            [node_group.node_group_template_id
             for node_group in cluster_template.node_groups]):
            template_users += [cluster_template.name]

    if cluster_users or template_users:
        raise ex.InvalidReferenceException(
            _("Node group template %(template)s is in use by "
              "cluster templates: %(users)s; and clusters: %(clusters)s") %
            {'template': node_group_template_id,
             'users': template_users and ', '.join(template_users) or 'N/A',
             'clusters': cluster_users and ', '.join(cluster_users) or 'N/A'})


def check_node_group_template_update(node_group_template_id, data, **kwargs):
    plugin_version = 'hadoop_version'
    if data.get('plugin_version'):
        plugin_version = 'plugin_version'

    if data.get('plugin_name') and not data.get(plugin_version):
        raise ex.InvalidReferenceException(
            _("You must specify a %s value "
              "for your plugin_name") % plugin_version)

    if data.get('plugin_name'):
        plugin = data.get('plugin_name')
        version = data.get(plugin_version)
        b.check_plugin_name_exists(plugin)
        b.check_plugin_supports_version(plugin, version)
    else:
        ngt = api.get_node_group_template(node_group_template_id)
        plugin = ngt.plugin_name
        if data.get(plugin_version):
            version = data.get(plugin_version)
            b.check_plugin_supports_version(plugin, version)
        else:
            version = ngt.hadoop_version

    if data.get('image_id'):
        b.check_image_registered(data['image_id'])
        b.check_required_image_tags(plugin,
                                    version,
                                    data['image_id'])

    b.check_node_group_basic_fields(plugin, version, data)

    if data.get('shares'):
        shares.check_shares(data['shares'])



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_config import cfg
from oslo_log import log as logging
import six
import six.moves.urllib.parse as urlparse
from stevedore import enabled

from sahara import conductor as cond
from sahara import exceptions as ex
from sahara.i18n import _

conductor = cond.API

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class DataSourceManager(object):
    def __init__(self):
        self.data_sources = {}
        self._load_data_sources()

    def _load_data_sources(self):
        config_ds = CONF.data_source_types
        extension_manager = enabled.EnabledExtensionManager(
            check_func=lambda ext: ext.name in config_ds,
            namespace='sahara.data_source.types',
            invoke_on_load=True
        )

        for ext in extension_manager.extensions:
            if ext.name in self.data_sources:
                raise ex.ConfigurationError(
                    _("Data source with name '%s' already exists.") %
                    ext.name)
            ext.obj.name = ext.name
            self.data_sources[ext.name] = ext.obj
            LOG.info("Data source name {ds_name} loaded {entry_point}".format(
                ds_name=ext.name, entry_point=ext.entry_point_target))

        if len(self.data_sources) < len(config_ds):
            loaded_ds = set(six.iterkeys(self.data_sources))
            requested_ds = set(config_ds)
            raise ex.ConfigurationError(
                _("Data sources couldn't be loaded: %s") %
                ", ".join(requested_ds - loaded_ds))

    def get_data_sources(self):
        config_ds = CONF.data_source_types
        return [self.get_data_source(name).name for name in config_ds]

    def get_data_source(self, name):
        res = self.data_sources.get(name)
        if res is None:
            raise ex.InvalidDataException(_("Invalid data source"))
        return res

    def get_data_source_by_url(self, url):
        url = urlparse.urlparse(url)
        if not url.scheme:
            raise ex.InvalidDataException(_("Data source url must have a"
                                            " scheme"))
        return self.get_data_source(url.scheme)


DATA_SOURCES = None


def setup_data_sources():
    global DATA_SOURCES
    DATA_SOURCES = DataSourceManager()



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import six
import testtools

from sahara import exceptions
from sahara.service.api import v10 as api
from sahara.service.validations import clusters as c
from sahara.service.validations import clusters_schema as c_schema
from sahara.tests.unit import base
from sahara.tests.unit.service.validation import utils as u


class TestClusterCreateValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestClusterCreateValidation, self).setUp()
        self._create_object_fun = c.check_cluster_create
        self.scheme = c_schema.CLUSTER_SCHEMA
        api.plugin_base.setup_plugins()

    def test_cluster_create_v_plugin_vers(self):
        self._assert_create_object_validation(
            data={
                'name': 'testname',
                'plugin_name': 'fake',
                'hadoop_version': '1'
            },
            bad_req_i=(1, "INVALID_REFERENCE",
                       "Requested plugin 'fake' "
                       "doesn't support version '1'"),
        )

    def test_cluster_create_v_required(self):
        self._assert_create_object_validation(
            data={},
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'name' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': 'test-name'
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'plugin_name' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': 'testname',
                'plugin_name': 'fake'
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'hadoop_version' is a required property")
        )

    def test_cluster_create_v_types(self):
        data = {
            'name': "testname",
            'plugin_name': "fake",
            'hadoop_version': "0.1"
        }
        self._assert_types(data)

    def test_cluster_create_v_name_base(self):
        data = {
            'name': "testname",
            'plugin_name': "fake",
            'hadoop_version': "0.1"
        }
        self._assert_valid_name_hostname_validation(data)

    def test_cluster_create_v_unique_cl(self):
        data = {
            'name': 'test',
            'plugin_name': 'fake',
            'hadoop_version': '0.1'
        }
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, 'NAME_ALREADY_EXISTS',
                       "Cluster with name 'test' already exists")
        )

    def test_cluster_create_v_keypair_exists(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'wrong_keypair'
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "Requested keypair 'wrong_keypair' not found")
        )

    def test_cluster_create_v_keypair_type(self):
        self._assert_create_object_validation(
            data={
                'name': "test-name",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': '!'},
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "user_keypair_id: '!' is not a 'valid_keypair_name'")
        )

    def test_cluster_create_v_image_exists(self):
        self._assert_create_object_validation(
            data={
                'name': "test-name",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'default_image_id': '550e8400-e29b-41d4-a616-446655440000'
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Requested image '550e8400-e29b-41d4-a616-446655440000'"
                       " is not registered")
        )

    def test_cluster_create_v_plugin_name_exists(self):
        self._assert_create_object_validation(
            data={
                'name': "test-name",
                'plugin_name': "wrong_plugin",
                'hadoop_version': "0.1",
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Sahara doesn't contain plugin "
                       "with name 'wrong_plugin'")
        )

    def test_cluster_create_v_wrong_network(self):
        self._assert_create_object_validation(
            data={
                'name': "test-name",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': '53a36917-ab9f-4589-'
                                              '94ce-b6df85a68332'
            },
            bad_req_i=(1, 'NOT_FOUND', "Network 53a36917-ab9f-4589-"
                                       "94ce-b6df85a68332 not found")
        )

    def test_cluster_create_v_missing_network(self):
        self._assert_create_object_validation(
            data={
                'name': "test-name",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "'neutron_management_network' field is not found")
        )

    def test_cluster_create_v_long_instance_names(self):
        self._assert_create_object_validation(
            data={
                'name': "long-long-cluster-name",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "long-long-long-very-long-node-group-name",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                    }
                ]
            },
            bad_req_i=(1, 'INVALID_DATA',
                       "Composite hostname long-long-cluster-name-long-long-"
                       "long-very-long-node-group-name-100.novalocal "
                       "in provisioned cluster exceeds maximum limit 64 "
                       "characters")
        )

    def test_cluster_create_v_cluster_configs(self):
        self._assert_cluster_configs_validation(True)

    def test_cluster_create_v_right_data(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'cluster_configs': {
                    'general': {
                        u'Enable NTP service': True
                    }
                },
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca'
            }
        )

    def test_cluster_create_v_default_image_required_tags(self):
        self._assert_cluster_default_image_tags_validation()

    def test_cluster_create_security_groups(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "nodegroup",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group2'],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca'
                    }
                ]
            }
        )

    def test_cluster_create_missing_floating_pool(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "ng1",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group2'],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca'
                    },
                    {
                        "name": "ng2",
                        "node_processes": ["datanode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group2']
                    }
                ]
            }
        )

    def test_cluster_create_with_proxy_gateway(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "ng1",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group2'],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
                        "is_proxy_gateway": True
                    },
                    {
                        "name": "ng2",
                        "node_processes": ["datanode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group2']
                    }
                ]
            }
        )

    def test_cluster_create_security_groups_by_ids(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "nodegroup",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['2', '3'],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca'
                    }
                ]
            }
        )

    def test_cluster_missing_security_groups(self):
        self._assert_create_object_validation(
            data={
                'name': "testname",
                'plugin_name': "fake",
                'hadoop_version': "0.1",
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        "name": "nodegroup",
                        "node_processes": ["namenode"],
                        "flavor_id": "42",
                        "count": 100,
                        'security_groups': ['group1', 'group3'],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca'
                    }
                ]
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "Security group 'group3' not found")
        )

    def test_cluster_create_availability_zone(self):
        self._assert_create_object_validation(
            data={
                'name': 'testname',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        'name': 'nodegroup',
                        'node_processes': ['namenode'],
                        'flavor_id': '42',
                        'count': 100,
                        'security_groups': [],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
                        'availability_zone': 'nova',
                        'volumes_per_node': 1,
                        'volumes_size': 1,
                        'volumes_availability_zone': 'nova'
                    }
                ]
            }
        )

    def test_cluster_create_wrong_availability_zone(self):
        self._assert_create_object_validation(
            data={
                'name': 'testname',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        'name': 'nodegroup',
                        'node_processes': ['namenode'],
                        'flavor_id': '42',
                        'count': 100,
                        'security_groups': [],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
                        'availability_zone': 'nonexistent'
                    }
                ]
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "Nova availability zone 'nonexistent' not found")
        )

    def test_cluster_create_wrong_volumes_availability_zone(self):
        self._assert_create_object_validation(
            data={
                'name': 'testname',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'user_keypair_id': 'test_keypair',
                'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
                'neutron_management_network': 'd9a3bebc-f788-4b81-'
                                              '9a93-aa048022c1ca',
                'node_groups': [
                    {
                        'name': 'nodegroup',
                        'node_processes': ['namenode'],
                        'flavor_id': '42',
                        'count': 100,
                        'security_groups': [],
                        'floating_ip_pool':
                            'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
                        'volumes_per_node': 1,
                        'volumes_availability_zone': 'nonexistent'
                    }
                ]
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "Cinder availability zone 'nonexistent' not found")
        )


class TestClusterCreateFlavorValidation(base.SaharaWithDbTestCase):
    """Tests for valid flavor on cluster create.

    The following use cases for flavors during cluster create are validated:
      * Flavor id defined in a node group template and used in a cluster
        template.
      * Flavor id defined in node groups on cluster create.
      * Both node groups and cluster template defined on cluster create.
      * Node groups with node group template defined on cluster create.
    """

    def setUp(self):
        super(TestClusterCreateFlavorValidation, self).setUp()
        self.override_config('plugins', ['fake'])
        modules = [
            "sahara.service.validations.base.check_plugin_name_exists",
            "sahara.service.validations.base.check_plugin_supports_version",
            "sahara.service.validations.base._get_plugin_configs",
            "sahara.service.validations.base.check_node_processes",
        ]
        self.patchers = []
        for module in modules:
            patch = mock.patch(module)
            patch.start()
            self.patchers.append(patch)

        nova_p = mock.patch("sahara.utils.openstack.nova.client")
        nova = nova_p.start()
        self.patchers.append(nova_p)
        nova().flavors.list.side_effect = u._get_flavors_list
        api.plugin_base.setup_plugins()

    def tearDown(self):
        u.stop_patch(self.patchers)
        super(TestClusterCreateFlavorValidation, self).tearDown()

    def _create_node_group_template(self, flavor='42'):
        ng_tmpl = {
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "node_processes": ["namenode"],
            "name": "master",
            "flavor_id": flavor
        }
        return api.create_node_group_template(ng_tmpl).id

    def _create_cluster_template(self, ng_id):
        cl_tmpl = {
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "node_groups": [
                {"name": "master",
                 "count": 1,
                 "node_group_template_id": "%s" % ng_id}
            ],
            "name": "test-template"
        }
        return api.create_cluster_template(cl_tmpl).id

    def test_cluster_create_v_correct_flavor(self):
        ng_id = self._create_node_group_template(flavor='42')
        ctmpl_id = self._create_cluster_template(ng_id)

        data = {
            "name": "testname",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "cluster_template_id": '%s' % ctmpl_id,
            "neutron_management_network": "d9a3bebc-f788-4b81-"
                                          "9a93-aa048022c1ca",
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        patchers = u.start_patch(False)
        c.check_cluster_create(data)
        u.stop_patch(patchers)

        data1 = {
            "name": "testwithnodegroups",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "neutron_management_network": "d9a3bebc-f788-4b81-"
                                          "9a93-aa048022c1ca",
            "node_groups": [
                {
                    "name": "allinone",
                    "count": 1,
                    "flavor_id": "42",
                    "node_processes": [
                        "namenode",
                        "jobtracker",
                        "datanode",
                        "tasktracker"
                    ]
                }
            ],
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        patchers = u.start_patch(False)
        c.check_cluster_create(data1)
        u.stop_patch(patchers)

    def test_cluster_create_v_invalid_flavor(self):
        ng_id = self._create_node_group_template(flavor='10')
        ctmpl_id = self._create_cluster_template(ng_id)

        data = {
            "name": "testname",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "cluster_template_id": '%s' % ctmpl_id,
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        data1 = {
            "name": "testwithnodegroups",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "neutron_management_network": "d9a3bebc-f788-4b81-"
                                          "9a93-aa048022c1ca",
            "node_groups": [
                {
                    "name": "allinone",
                    "count": 1,
                    "flavor_id": "10",
                    "node_processes": [
                        "namenode",
                        "resourcemanager",
                        "datanode",
                        "nodemanager"
                    ]
                }
            ],
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        for values in [data, data1]:
            with testtools.ExpectedException(
                    exceptions.NotFoundException):
                patchers = u.start_patch(False)
                try:
                    c.check_cluster_create(values)
                except exceptions.NotFoundException as e:
                    message = six.text_type(e).split('\n')[0]
                    self.assertEqual("Requested flavor '10' not found",
                                     message)
                    raise e
                finally:
                    u.stop_patch(patchers)

    def test_cluster_create_cluster_tmpl_node_group_mixin(self):
        ng_id = self._create_node_group_template(flavor='10')
        ctmpl_id = self._create_cluster_template(ng_id)

        data = {
            "name": "testtmplnodegroups",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "cluster_template_id": '%s' % ctmpl_id,
            "neutron_management_network": "d9a3bebc-f788-4b81-"
                                          "9a93-aa048022c1ca",
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000',
            "node_groups": [
                {
                    "name": "allinone",
                    "count": 1,
                    "flavor_id": "42",
                    "node_processes": [
                        "namenode",
                        "resourcemanager",
                        "datanode",
                        "nodemanager"
                    ]
                }
            ]
        }
        patchers = u.start_patch(False)
        c.check_cluster_create(data)
        u.stop_patch(patchers)

    def test_cluster_create_node_group_tmpl_mixin(self):
        ng_id = self._create_node_group_template(flavor='23')
        data = {
            "name": "testtmplnodegroups",
            "plugin_name": "fake",
            "hadoop_version": "0.1",
            "neutron_management_network": "d9a3bebc-f788-4b81-"
                                          "9a93-aa048022c1ca",
            "node_groups": [
                {
                    "node_group_template_id": '%s' % ng_id,
                    "name": "allinone",
                    "count": 1,
                    "flavor_id": "42",
                    "node_processes": [
                        "namenode",
                        "resourcemanager",
                        "datanode",
                        "nodemanager"
                    ]
                },
            ],
            'default_image_id': '550e8400-e29b-41d4-a716-446655440000'
        }
        with testtools.ExpectedException(exceptions.NotFoundException):
            patchers = u.start_patch(False)
            try:
                c.check_cluster_create(data)
            except exceptions.NotFoundException as e:
                message = six.text_type(e).split('\n')[0]
                self.assertEqual("Requested flavor '23' not found",
                                 message)
                raise e
            finally:
                u.stop_patch(patchers)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Provides means to wrap dicts coming from DB layer in objects.

The Conductor can fetch only values represented by JSON.
That limitation comes from Oslo RPC implementation.
This module provides means to wrap a fetched value, always
dictionary, into an immutable Resource object. A descendant of
Resource class might provide back references to parent objects
and helper methods.
"""

import datetime

import six

from sahara.conductor import objects
from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp import s3_common
from sahara.swift import swift_helper
from sahara.utils import types


def wrap(resource_class):
    """A decorator wraps dict returned by a given function into a Resource."""

    def decorator(func):
        def handle(*args, **kwargs):
            ret = func(*args, **kwargs)
            if isinstance(ret, types.Page):
                return types.Page([resource_class(el) for el in ret],
                                  ret.prev, ret.next)
            elif isinstance(ret, list):
                return [resource_class(el) for el in ret]
            elif ret:
                return resource_class(ret)
            else:
                return None

        return handle

    return decorator


class Resource(types.FrozenDict):
    """Represents dictionary as an immutable object.

    Enhancing it with back references and helper methods.

    For instance, the following dictionary:
    {'first': {'a': 1, 'b': 2}, 'second': [1,2,3]}

    after wrapping with Resource will look like an object, let it be
    'res' with the following fields:
    res.first
    res.second

    'res.first' will in turn be wrapped into Resource with two fields:
    res.first.a == 1
    res.first.b == 2

    'res.second', which is a list, will be transformed into a tuple
    for immutability:
    res.second == (1,2,3)

    Additional helper methods could be specified in descendant
    classes. '_children' specifies children of that specific Resource
    in the following format: {refname: (child_class, backref_name)}
    Back reference is a reference to parent object which is
    injected into a Resource during wrapping.
    """

    _resource_name = 'resource'
    _children = {}
    _filter_fields = []
    _sanitize_fields = {}

    def __init__(self, dct):
        super(Resource, self).__setattr__('_initial_dict', dct)
        newdct = dict()
        for refname, entity in six.iteritems(dct):
            newdct[refname] = self._wrap_entity(refname, entity)

        super(Resource, self).__init__(newdct)

    def to_dict(self):
        """Return dictionary representing the Resource for REST API.

        On the way filter out fields which shouldn't be exposed.
        """
        return self._to_dict(None)

    def to_wrapped_dict(self):
        return {self._resource_name: self.to_dict()}

    # Construction

    def _wrap_entity(self, refname, entity):
        if isinstance(entity, Resource):
            # that is a back reference
            return entity
        elif isinstance(entity, list):
            return self._wrap_list(refname, entity)
        elif isinstance(entity, dict):
            return self._wrap_dict(refname, entity)
        elif self._is_passthrough_type(entity):
            return entity
        else:
            raise TypeError(_("Unsupported type: %s") % type(entity).__name__)

    def _wrap_list(self, refname, lst):
        newlst = [self._wrap_entity(refname, entity) for entity in lst]

        return types.FrozenList(newlst)

    def _wrap_dict(self, refname, dct):
        if refname in self._children:
            dct = dict(dct)
            child_class = self._children[refname][0]
            backref_name = self._children[refname][1]
            if backref_name:
                dct[backref_name] = self
            return child_class(dct)
        else:
            return Resource(dct)

    def _is_passthrough_type(self, entity):
        return (entity is None or
                isinstance(entity,
                           (six.integer_types, float,
                            datetime.datetime, six.string_types)))

    # Conversion to dict

    def _to_dict(self, backref):
        dct = dict()
        for refname, entity in six.iteritems(self):
            if refname != backref and refname not in self._filter_fields:
                childs_backref = None
                if refname in self._children:
                    childs_backref = self._children[refname][1]
                dct[refname] = self._entity_to_dict(entity, childs_backref)
                sanitize = self._sanitize_fields.get(refname)
                if sanitize is not None:
                    dct[refname] = sanitize(self, dct[refname])
        return dct

    def _entity_to_dict(self, entity, childs_backref):
        if isinstance(entity, Resource):
            return entity._to_dict(childs_backref)
        elif isinstance(entity, list):
            return self._list_to_dict(entity, childs_backref)
        elif entity is not None:
            return entity

    def _list_to_dict(self, lst, childs_backref):
        return [self._entity_to_dict(entity, childs_backref) for entity in lst]

    def __getattr__(self, item):
        return self[item]

    def __setattr__(self, *args):
        raise ex.FrozenClassError(self)


class NodeGroupTemplateResource(Resource, objects.NodeGroupTemplate):
    _resource_name = 'node_group_template'


class InstanceResource(Resource, objects.Instance):
    _filter_fields = ['tenant_id', 'node_group_id', "volumes"]

    @property
    def cluster_id(self):
        return self.node_group.cluster_id

    @property
    def cluster(self):
        return self.node_group.cluster


class NodeGroupResource(Resource, objects.NodeGroup):
    _children = {
        'instances': (InstanceResource, 'node_group'),
        'node_group_template': (NodeGroupTemplateResource, None)
    }

    _filter_fields = ['tenant_id', 'cluster_id', 'cluster_template_id',
                      'image_username', 'open_ports']


class ClusterTemplateResource(Resource, objects.ClusterTemplate):
    _resource_name = 'cluster_template'

    _children = {
        'node_groups': (NodeGroupResource, 'cluster_template')
    }


class ClusterHealthCheckResource(Resource, objects.ClusterHealthCheck):
    _resource_name = 'cluster_health_check'


class ClusterVerificationResource(Resource, objects.ClusterVerification):
    _resource_name = 'cluster_verification'
    _children = {
        'checks': (ClusterHealthCheckResource, 'verification')
    }


class ClusterResource(Resource, objects.Cluster):

    def sanitize_cluster_configs(self, cluster_configs):
        if 'proxy_configs' in cluster_configs:
            del cluster_configs['proxy_configs']
        return cluster_configs

    _resource_name = 'cluster'

    _children = {
        'node_groups': (NodeGroupResource, 'cluster'),
        'cluster_template': (ClusterTemplateResource, None),
        'verification': (ClusterVerificationResource, 'cluster')
    }

    _filter_fields = ['management_private_key', 'extra', 'rollback_info',
                      'sahara_info']
    _sanitize_fields = {'cluster_configs': sanitize_cluster_configs}


class ImageResource(Resource, objects.Image):

    _resource_name = 'image'

    @property
    def dict(self):
        return self.to_dict()

    @property
    def wrapped_dict(self):
        return {'image': self.dict}

    def _sanitize_image_properties(self, image_props):
        if 'links' in image_props:
            del image_props['links']
        return image_props

    _sanitize_fields = {'links': _sanitize_image_properties}


# EDP Resources

class DataSource(Resource, objects.DataSource):
    _resource_name = "data_source"
    _filter_fields = ['credentials']


class JobExecution(Resource, objects.JobExecution):

    def sanitize_job_configs(self, job_configs):
        if 'configs' in job_configs:
            configs = job_configs['configs']
            if swift_helper.HADOOP_SWIFT_USERNAME in configs:
                configs[swift_helper.HADOOP_SWIFT_USERNAME] = ""
            if swift_helper.HADOOP_SWIFT_PASSWORD in configs:
                configs[swift_helper.HADOOP_SWIFT_PASSWORD] = ""
            if s3_common.S3_ACCESS_KEY_CONFIG in configs:
                configs[s3_common.S3_ACCESS_KEY_CONFIG] = ""
            if s3_common.S3_SECRET_KEY_CONFIG in configs:
                configs[s3_common.S3_SECRET_KEY_CONFIG] = ""

        if 'trusts' in job_configs:
            del job_configs['trusts']
        if 'proxy_configs' in job_configs:
            del job_configs['proxy_configs']
        return job_configs

    def sanitize_info(self, info):
        if 'actions' in info:
            for d in info['actions']:
                if 'conf' in d:
                    del d['conf']
        return info

    _resource_name = "job_execution"
    _filter_fields = ['extra']
    _sanitize_fields = {'job_configs': sanitize_job_configs,
                        'info': sanitize_info}
    # TODO(egafford): Sanitize interface ("secret" bool field on job args?)


class JobBinary(Resource, objects.JobBinary):
    _resource_name = "job_binary"
    _filter_fields = ['extra']


class JobBinaryInternal(Resource, objects.JobBinaryInternal):
    _resource_name = "job_binary_internal"


class Job(Resource, objects.Job):
    _resource_name = "job"
    _children = {
        'mains': (JobBinary, None),
        'libs': (JobBinary, None)
    }



#   Copyright 2017 Massachusetts Open Cloud
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import botocore.exceptions
import botocore.session
from oslo_config import cfg
import six

import sahara.exceptions as ex
from sahara.i18n import _
from sahara.service.castellan import utils as key_manager

S3_JB_PREFIX = "s3://"
S3_ACCESS_KEY_CONFIG = "fs.s3a.access.key"
S3_SECRET_KEY_CONFIG = "fs.s3a.secret.key"
S3_ENDPOINT_CONFIG = "fs.s3a.endpoint"
S3_BUCKET_IN_PATH_CONFIG = "fs.s3a.path.style.access"
S3_SSL_CONFIG = "fs.s3a.connection.ssl.enabled"
S3_DS_CONFIGS = [S3_ACCESS_KEY_CONFIG,
                 S3_SECRET_KEY_CONFIG,
                 S3_ENDPOINT_CONFIG,
                 S3_BUCKET_IN_PATH_CONFIG,
                 S3_SSL_CONFIG]
CONF = cfg.CONF


def _get_s3_client(extra):
    sess = botocore.session.get_session()
    secretkey = key_manager.get_secret(extra['secretkey'])
    return sess.create_client(
        's3',
        # TODO(jfreud): investigate region name support
        region_name=None,
        # TODO(jfreud): investigate configurable verify
        verify=False,
        endpoint_url=extra['endpoint'],
        aws_access_key_id=extra['accesskey'],
        aws_secret_access_key=secretkey
    )


def _get_names_from_job_binary_url(url):
    parse = six.moves.urllib.parse.urlparse(url)
    return (parse.netloc + parse.path).split('/', 1)


def _get_raw_job_binary_data(job_binary, conn):
    names = _get_names_from_job_binary_url(job_binary.url)
    bucket, obj = names
    try:
        size = conn.head_object(Bucket=bucket, Key=obj)['ContentLength']
        # We have bytes, but want kibibytes:
        total_KB = size / 1024.0
        if total_KB > CONF.job_binary_max_KB:
            raise ex.DataTooBigException(
                round(total_KB, 1), CONF.job_binary_max_KB,
                _("Size of S3 object (%(size)sKB) is greater "
                  "than maximum (%(maximum)sKB)"))
        body = conn.get_object(Bucket=bucket, Key=obj)['Body'].read()
    except ex.DataTooBigException:
        raise
    except Exception:
        raise ex.S3ClientException("Couldn't get object from s3")
    return body


def _validate_job_binary_url(job_binary_url):
    if not job_binary_url.startswith(S3_JB_PREFIX):
        # Sanity check
        raise ex.BadJobBinaryException(
            _("URL for binary in S3 must start with %s") % S3_JB_PREFIX)
    names = _get_names_from_job_binary_url(job_binary_url)
    if len(names) == 1:
        # we have a bucket instead of an individual object
        raise ex.BadJobBinaryException(
            _("URL for binary in S3 must specify an object not a bucket"))


def get_raw_job_binary_data(job_binary):
    _validate_job_binary_url(job_binary.url)
    try:
        conn = _get_s3_client(job_binary.extra)
    except Exception:
        raise ex.S3ClientException("Couldn't create boto client")
    return _get_raw_job_binary_data(job_binary, conn)



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import itertools

import mock

from sahara.service.validations.edp import job as j
from sahara.service.validations.edp import job_execution_schema as j_e_schema
from sahara.service.validations.edp import job_interface as j_i
from sahara.service.validations.edp import job_schema as j_schema
from sahara.tests.unit.service.validation import utils as u
from sahara.utils import edp


def _configs(**kwargs):
    arg = {
        "name": "Reducer Count",
        "mapping_type": "configs",
        "location": "mapred.reduce.tasks",
        "value_type": "number",
        "required": True,
        "default": "1"
    }
    arg.update(kwargs)
    return arg


def _params(**kwargs):
    arg = {
        "name": "Input Path",
        "mapping_type": "params",
        "location": "INPUT",
        "value_type": "data_source",
        "required": False,
        "default": "hdfs://path"
    }
    arg.update(kwargs)
    return arg


def _args(**kwargs):
    arg = {
        "name": "Positional Argument",
        "mapping_type": "args",
        "location": "0",
        "value_type": "string",
        "required": False,
        "default": "arg_value"
    }
    arg.update(kwargs)
    return arg


_mapping_types = {"configs", "args", "params"}


def _job(job_type, interface):
    return {"name": "job", "type": job_type, "interface": interface}


_job_types = [
    _job(edp.JOB_TYPE_HIVE, [_configs(), _params()]),
    _job(edp.JOB_TYPE_PIG, [_configs(), _params(), _args()]),
    _job(edp.JOB_TYPE_MAPREDUCE, [_configs()]),
    _job(edp.JOB_TYPE_MAPREDUCE_STREAMING, [_configs()]),
    _job(edp.JOB_TYPE_JAVA, [_configs(), _args()]),
    _job(edp.JOB_TYPE_SHELL, [_configs(), _params(), _args()]),
    _job(edp.JOB_TYPE_SPARK, [_configs(), _args()]),
    _job(edp.JOB_TYPE_STORM, [_args()]),
    _job(edp.JOB_TYPE_PYLEUS, [])
]


class TestJobInterfaceValidation(u.ValidationTestCase):

    def setUp(self):
        super(TestJobInterfaceValidation, self).setUp()
        self._create_object_fun = j.check_interface
        self.scheme = j_schema.JOB_SCHEMA

    def test_interface(self):
        for job in _job_types:
            self._assert_create_object_validation(data=job)

    def test_no_interface(self):
        job = _job(edp.JOB_TYPE_PIG, None)
        self._assert_create_object_validation(data=job)

    def test_overlapping_names(self):
        job = _job(edp.JOB_TYPE_PIG,
                   [_configs(), _configs(location="mapred.map.tasks")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Name must be unique within the interface for any job."))

    def test_unacceptable_types(self):
        for job in _job_types:
            acceptable_types = [arg['mapping_type']
                                for arg in job['interface']]
            unacceptable_types = _mapping_types - set(acceptable_types)
            unacceptable_args = (globals()["_" + m_type]()
                                 for m_type in unacceptable_types)
            bad_job = job.copy()
            for arg in unacceptable_args:
                bad_job['interface'] = [arg]
                permutations = itertools.permutations(acceptable_types)
                msgs = ["Only mapping types %s are allowed for job type %s." %
                        (list(permutation), bad_job['type'])
                        for permutation in permutations]
                self._assert_create_object_validation(
                    data=bad_job, bad_req_i=(1, "INVALID_DATA", msgs))

    def test_bad_positional_arg_locations(self):
        job = _job(edp.JOB_TYPE_PIG, [_args(location="1")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Locations of positional arguments must be an unbroken "
                "integer sequence ascending from 0."))

        job = _job(edp.JOB_TYPE_PIG, [_args(location="fish")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Locations of positional arguments must be an unbroken "
                "integer sequence ascending from 0."))

        job = _job(edp.JOB_TYPE_PIG,
                   [_args(), _args(location="2", name="Argument 2")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Locations of positional arguments must be an unbroken "
                "integer sequence ascending from 0."))

    def test_required_positional_arg_without_default(self):
        arg = _args(required=False)
        del arg['default']
        job = _job(edp.JOB_TYPE_PIG, [arg])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Positional arguments must be given default values if they "
                "are not required."))

    def test_overlapping_locations(self):
        job = _job(edp.JOB_TYPE_PIG,
                   [_configs(), _configs(name="Mapper Count")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "The combination of mapping type and location must be unique "
                "within the interface for any job."))

    def test_number_values(self):
        job = _job(edp.JOB_TYPE_PIG, [_configs(default="fish")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Value 'fish' is not a valid number."))

    @mock.patch('sahara.conductor.API.data_source_get')
    def test_data_source_values(self, data_source):
        data_source.return_value = True
        job = _job(edp.JOB_TYPE_PIG,
                   [_params(default="DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF")])
        self._assert_create_object_validation(data=job)

        data_source.return_value = False
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_REFERENCE",
                "DataSource with id 'DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF' "
                "doesn't exist"))

        job = _job(edp.JOB_TYPE_PIG, [_params(default="Filial Piety")])
        self._assert_create_object_validation(
            data=job, bad_req_i=(
                1, "INVALID_DATA",
                "Data source value 'Filial Piety' is neither a valid data "
                "source ID nor a valid URL."))

    def test_default_data_type(self):
        param = _params()
        del param['value_type']
        job = _job(edp.JOB_TYPE_PIG, [param])
        self._assert_create_object_validation(data=job)
        assert param['value_type'] == j_i.DEFAULT_DATA_TYPE


int_arg = collections.namedtuple("int_arg",
                                 ["name", "mapping_type", "location",
                                  "value_type", "required", "default"])


def j_e_i_wrapper(data):
    job = mock.Mock(
        interface=[int_arg(**_configs()),
                   int_arg(**_args()),
                   int_arg(**_params())]
    )
    j_i.check_execution_interface(data, job)


class TestJobExecutionInterfaceValidation(u.ValidationTestCase):

    def setUp(self):
        super(TestJobExecutionInterfaceValidation, self).setUp()
        self._create_object_fun = j_e_i_wrapper
        self.scheme = j_e_schema.JOB_EXEC_SCHEMA

    def test_valid_execution(self):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "2",
                              "Input Path": "swift://path",
                              "Positional Argument": "value"}}
        self._assert_create_object_validation(data=data)

    def test_no_execution_interface(self):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF"}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA",
            "An interface was specified with the template for this job. "
            "Please pass an interface map with this job (even if empty)."))

    def test_bad_argument_name(self):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Fish": "Rainbow Trout",
                              "Reducer Count": "2",
                              "Input Path": "swift://path"}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA", "Argument names: ['Fish'] were not found in "
                               "the interface for this job."))

    def test_required_argument_missing(self):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Positional Argument": "Value"}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA", "Argument names: ['Reducer Count'] are "
                               "required for this job."))

    @mock.patch('sahara.conductor.API.data_source_get')
    def test_bad_values(self, data_source):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "Two"}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA", "Value 'Two' is not a valid number."))

        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "2",
                              "Input Path": "not_a_url"}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA", "Data source value 'not_a_url' is neither a "
                               "valid data source ID nor a valid URL."))

        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "2",
                              "Positional Argument": 2}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA", "Value '2' is not a valid string."))

        data_source.return_value = False
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface":
                    {"Reducer Count": "2",
                     "Input Path": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF"}}
        self._assert_create_object_validation(
            data=data, bad_req_i=(
                1, "INVALID_REFERENCE",
                "DataSource with id 'DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF' "
                "doesn't exist"))

    def test_overlapping_data(self):
        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "2"},
                "job_configs": {"configs": {"mapred.reduce.tasks": "2"}}}
        self._assert_create_object_validation(data=data)

        data = {"cluster_id": "DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF",
                "interface": {"Reducer Count": "2"},
                "job_configs": {"configs": {"mapred.reduce.tasks": "3"}}}
        self._assert_create_object_validation(data=data, bad_req_i=(
            1, "INVALID_DATA",
            "Argument 'Reducer Count' was passed both through the interface "
            "and in location 'configs'.'mapred.reduce.tasks'. Please pass "
            "this through either the interface or the configuration maps, "
            "not both."))



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from sahara.service.api.v2 import images
from sahara.tests.unit import base


class TestImageApi(base.SaharaTestCase):
    def SetUp(self):
        super(TestImageApi, self).SetUp()

    @mock.patch('sahara.utils.openstack.images.SaharaImageManager')
    def test_get_image_tags(self, mock_manager):
        image = mock.Mock()
        manager = mock.Mock()
        manager.get.return_value = mock.Mock(tags=['foo', 'bar', 'baz'])
        mock_manager.return_value = manager
        self.assertEqual(['foo', 'bar', 'baz'], images.get_image_tags(image))

    @mock.patch('sahara.utils.openstack.images.SaharaImageManager')
    def test_set_image_tags(self, mock_manager):
        def _tag(image, to_add):
            return tags.append('qux')

        def _untag(image, to_remove):
            return tags.remove('bar')

        expected_tags = ['foo', 'baz', 'qux']
        tags = ['foo', 'bar', 'baz']
        image = mock.Mock()
        manager = mock.Mock()
        manager.get.return_value = mock.Mock(tags=tags)
        manager.tag.side_effect = _tag
        manager.untag.side_effect = _untag
        mock_manager.return_value = manager

        self.assertEqual(expected_tags,
                         images.set_image_tags(image, expected_tags).tags)

    @mock.patch('sahara.utils.openstack.images.SaharaImageManager')
    def test_remove_image_tags(self, mock_manager):
        def _untag(image, to_remove):
            for i in range(len(to_remove)):
                actual_tags.pop()
            return actual_tags

        actual_tags = ['foo', 'bar', 'baz']
        image = mock.Mock()
        manager = mock.Mock()
        manager.get.return_value = mock.Mock(tags=actual_tags)
        manager.untag.side_effect = _untag
        mock_manager.return_value = manager

        self.assertEqual([], images.remove_image_tags(image).tags)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
from oslo_utils import uuidutils

from sahara import conductor as cond
from sahara.utils import edp


conductor = cond.API

_java_main_class = "org.apache.hadoop.examples.WordCount"
_java_opts = "-Dparam1=val1 -Dparam2=val2"


def create_job_exec(type, configs=None, proxy=False, info=None):
    b = create_job_binary('1', type)
    j = _create_job('2', b, type)
    _cje_func = _create_job_exec_with_proxy if proxy else _create_job_exec
    e = _cje_func(j.id, type, configs, info)
    return j, e


def _create_job(id, job_binary, type):
    job = mock.Mock()
    job.id = id
    job.type = type
    job.name = 'special_name'
    job.interface = []
    if edp.compare_job_type(type, edp.JOB_TYPE_PIG, edp.JOB_TYPE_HIVE):
        job.mains = [job_binary]
        job.libs = None
    else:
        job.libs = [job_binary]
        job.mains = None
    return job


def create_job_binary(id, type):
    binary = mock.Mock()
    binary.id = id
    binary.url = "internal-db://42"
    if edp.compare_job_type(type, edp.JOB_TYPE_PIG):
        binary.name = "script.pig"
    elif edp.compare_job_type(type, edp.JOB_TYPE_MAPREDUCE, edp.JOB_TYPE_JAVA):
        binary.name = "main.jar"
    else:
        binary.name = "script.q"
    return binary


def create_cluster(plugin_name='fake', hadoop_version='0.1'):
    cluster = mock.Mock()
    cluster.plugin_name = plugin_name
    cluster.hadoop_version = hadoop_version
    return cluster


def create_data_source(url, name=None, id=None):
    data_source = mock.Mock()
    data_source.url = url
    if url.startswith("swift"):
        data_source.type = "swift"
        data_source.credentials = {'user': 'admin',
                                   'password': 'admin1'}
    elif url.startswith("hdfs"):
        data_source.type = "hdfs"
    if name is not None:
        data_source.name = name
    if id is not None:
        data_source.id = id
    return data_source


def _create_job_exec(job_id, type, configs=None, info=None):
    j_exec = mock.Mock()
    j_exec.id = uuidutils.generate_uuid()
    j_exec.job_id = job_id
    j_exec.job_configs = configs
    j_exec.info = info
    j_exec.input_id = 4
    j_exec.output_id = 5
    j_exec.engine_job_id = None
    j_exec.data_source_urls = {}
    if not j_exec.job_configs:
        j_exec.job_configs = {}
    if edp.compare_job_type(type, edp.JOB_TYPE_JAVA):
        j_exec.job_configs['configs']['edp.java.main_class'] = _java_main_class
        j_exec.job_configs['configs']['edp.java.java_opts'] = _java_opts
    return j_exec


def _create_job_exec_with_proxy(job_id, type, configs=None, info=None):
    j_exec = _create_job_exec(job_id, type, configs)
    j_exec.id = '00000000-1111-2222-3333-4444444444444444'
    j_exec.info = info
    j_exec.job_configs['proxy_configs'] = {
        'proxy_username': 'job_' + j_exec.id,
        'proxy_password': '55555555-6666-7777-8888-999999999999',
        'proxy_trust_id': '0123456789abcdef0123456789abcdef'
        }
    return j_exec



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from oslo_concurrency import processutils
import paramiko
import six

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.utils import tempfiles


def to_paramiko_private_key(pkey):
    """Convert private key (str) to paramiko-specific RSAKey object."""
    return paramiko.RSAKey(file_obj=six.StringIO(pkey))


def generate_key_pair(key_length=2048):
    """Create RSA key pair with specified number of bits in key.

    Returns tuple of private and public keys.
    """
    with tempfiles.tempdir() as tmpdir:
        keyfile = os.path.join(tmpdir, 'tempkey')
        # The key is generated in the old PEM format, instead of the native
        # format of OpenSSH >=6.5, because paramiko does not support it:
        # https://github.com/paramiko/paramiko/issues/602
        args = [
            'ssh-keygen',
            '-q',  # quiet
            '-N', '',  # w/o passphrase
            '-m', 'PEM',  # old PEM format
            '-t', 'rsa',  # create key of rsa type
            '-f', keyfile,  # filename of the key file
            '-C', 'Generated-by-Sahara'  # key comment
        ]
        if key_length is not None:
            args.extend(['-b', key_length])
        processutils.execute(*args)
        if not os.path.exists(keyfile):
            raise ex.SystemError(_("Private key file hasn't been created"))
        with open(keyfile) as keyfile_fd:
            private_key = keyfile_fd.read()
        public_key_path = keyfile + '.pub'
        if not os.path.exists(public_key_path):
            raise ex.SystemError(_("Public key file hasn't been created"))
        with open(public_key_path) as public_key_path_fd:
            public_key = public_key_path_fd.read()

        return private_key, public_key



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.utils import patches
patches.patch_minidom_writexml()

import os
import sys

# If ../sahara/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                                os.pardir,
                                                os.pardir))
if os.path.exists(os.path.join(possible_topdir,
                               'sahara',
                               '__init__.py')):
    sys.path.insert(0, possible_topdir)


import sahara.main as server


def setup_api():
    server.setup_common(possible_topdir, 'API')

    app = server.make_app()

    server.setup_sahara_api('distributed')
    server.setup_auth_policy()
    return app


def main():
    app = setup_api()

    launcher = server.get_process_launcher()
    api_service = server.SaharaWSGIService("sahara-api", app)
    server.launch_api_service(launcher, api_service)



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import testtools

import sahara.exceptions as ex
from sahara.service.edp.data_sources.hdfs.implementation import HDFSType
from sahara.tests.unit import base


class TestHDFSType(base.SaharaTestCase):
    def setUp(self):
        super(TestHDFSType, self).setUp()
        self.hdfs_type = HDFSType()

    def test_hdfs_type_validation_wrong_schema(self):
        data = {
            "name": "test_data_data_source",
            "url": "hdf://test_cluster/",
            "type": "hdfs",
            "description": "incorrect url schema"
        }
        with testtools.ExpectedException(ex.InvalidDataException):
            self.hdfs_type.validate(data)

    def test_hdfs_type_validation_correct_url(self):
        data = {
            "name": "test_data_data_source",
            "url": "hdfs://test_cluster/",
            "type": "hdfs",
            "description": "correct url schema"
        }
        self.hdfs_type.validate(data)

    def test_hdfs_type_validation_local_rel_url(self):
        data = {
            "name": "test_data_data_source",
            "url": "mydata/input",
            "type": "hdfs",
            "description": "correct url schema for relative path on local hdfs"
        }
        self.hdfs_type.validate(data)

    def test_hdfs_type_validation_local_abs_url(self):
        data = {
            "name": "test_data_data_source",
            "url": "/tmp/output",
            "type": "hdfs",
            "description": "correct url schema for absolute path on local hdfs"
        }
        self.hdfs_type.validate(data)

    @mock.patch('sahara.service.edp.data_sources.hdfs.implementation.h')
    def test_prepare_cluster(self, mock_h):
        cluster = mock.Mock()
        data_source = mock.Mock()
        runtime_url = "runtime_url"
        mock_h.configure_cluster_for_hdfs = mock.Mock()

        self.hdfs_type.prepare_cluster(data_source, cluster,
                                       runtime_url=runtime_url)

        mock_h.configure_cluster_for_hdfs.assert_called_once_with(cluster,
                                                                  runtime_url)



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import swiftclient

from sahara import context
from sahara.swift import swift_helper as sh
from sahara.swift import utils as su
from sahara.utils.openstack import base
from sahara.utils.openstack import keystone as k

opts = [
    cfg.BoolOpt('api_insecure',
                default=False,
                help='Allow to perform insecure SSL requests to swift.'),
    cfg.StrOpt('ca_file',
               help='Location of ca certificates file to use for swift '
                    'client requests.'),
    cfg.StrOpt("endpoint_type",
               default="internalURL",
               help="Endpoint type for swift client requests")
]

swift_group = cfg.OptGroup(name='swift',
                           title='Swift client options')

CONF = cfg.CONF
CONF.register_group(swift_group)
CONF.register_opts(opts, group=swift_group)


def client(username, password, trust_id=None):
    '''return a Swift client

    This will return a Swift client for the specified username scoped to the
    current context project, unless a trust identifier is specified.

    If a trust identifier is present then the Swift client will be created
    based on a preauthorized token generated by the username scoped to the
    trust identifier.

    :param username: The username for the Swift client
    :param password: The password associated with the username
    :param trust_id: A trust identifier for scoping the username (optional)
    :returns: A Swift client object

    '''
    if trust_id:
        proxyauth = k.auth_for_proxy(username, password, trust_id)
        return client_from_token(k.token_from_auth(proxyauth))
    else:
        return swiftclient.Connection(
            auth_version='3',
            cacert=CONF.swift.ca_file,
            insecure=CONF.swift.api_insecure,
            authurl=su.retrieve_auth_url(CONF.keystone.endpoint_type),
            user=username,
            key=password,
            tenant_name=sh.retrieve_tenant(),
            retries=CONF.retries.retries_number,
            retry_on_ratelimit=True,
            starting_backoff=CONF.retries.retry_after,
            max_backoff=CONF.retries.retry_after)


def client_from_token(token=None):
    if not token:
        token = context.get_auth_token()
    '''return a Swift client authenticated from a token.'''
    return swiftclient.Connection(auth_version='3',
                                  cacert=CONF.swift.ca_file,
                                  insecure=CONF.swift.api_insecure,
                                  preauthurl=base.url_for(
                                      service_type="object-store",
                                      endpoint_type=CONF.swift.endpoint_type),
                                  preauthtoken=token,
                                  retries=CONF.retries.retries_number,
                                  retry_on_ratelimit=True,
                                  starting_backoff=CONF.retries.retry_after,
                                  max_backoff=CONF.retries.retry_after)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.tests.unit import base
from sahara.utils import resources


class SimpleResourceTestCase(base.SaharaTestCase):

    def setUp(self):
        super(SimpleResourceTestCase, self).setUp()
        self.test_name = "test_res"
        self.test_info_0 = {"a": "a"}
        self.test_info_1 = {"b": "b"}

    def test_resource_init_attrs(self):
        r = resources.Resource(_name=self.test_name, _info=self.test_info_0)
        r.b = "b"

        self.assertEqual("a", r.a)
        self.assertEqual("b", r.__getattr__("b"))
        self.assertIn("b", r.__dict__)

        self.assertEqual(self.test_info_0, r._info)
        self.assertEqual(self.test_name, r._name)
        self.assertEqual(self.test_name, r.__resource_name__)

    def test_resource_to_dict(self):
        r = resources.Resource(_name=self.test_name, _info=self.test_info_0)

        self.assertEqual(self.test_info_0, r.to_dict())
        self.assertEqual({self.test_name: self.test_info_0}, r.wrapped_dict)

    def test_resource_eq(self):
        r0 = resources.Resource(_name=self.test_name, _info=self.test_info_0)
        r1 = resources.Resource(_name=self.test_name, _info=self.test_info_1)

        self.assertNotEqual(r0, r1)

    def test_as_resource(self):
        r = resources.Resource(_name=self.test_name, _info=self.test_info_0)
        self.assertEqual(r, r.as_resource())

    def test_repr(self):
        r = resources.Resource(_name=self.test_name, _info=self.test_info_0)

        dict_repr = self.test_info_0.__repr__()
        self.assertEqual("<test_res %s>" % dict_repr, r.__repr__())


class InheritedBaseResourceTestCase(base.SaharaTestCase):

    def test_to_dict_no_filters(self):
        class A(resources.BaseResource):
            __filter_cols__ = []

        test_a = A()
        test_a.some_attr = "some_value"
        a_dict = test_a.to_dict()

        self.assertEqual({"some_attr": "some_value"}, a_dict)

    def test_to_dict_with_filters_and_sa(self):
        class A(resources.BaseResource):
            __filter_cols__ = ["filtered"]

        test_a = A()
        test_a.some_attr = "some_value"
        test_a.filtered = "something_hidden"
        test_a._sa_instance_state = "some_sqlalchemy_magic"
        a_dict = test_a.to_dict()

        self.assertEqual({"some_attr": "some_value"}, a_dict)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import functools

from oslo_config import cfg
import six
import swiftclient

import sahara.exceptions as ex
from sahara.i18n import _
from sahara.service.castellan import utils as key_manager
from sahara.swift import utils as su
from sahara.utils.openstack import swift as sw


CONF = cfg.CONF


def _get_names_from_url(url):
    parse = six.moves.urllib.parse.urlparse(url)
    return (parse.netloc + parse.path).split('/', 1)


def _get_raw_data(job_binary, conn):
    names = _get_names_from_url(job_binary.url)
    container, obj = names

    # if container name has '.sahara' suffix we need to strip it
    if container.endswith(su.SWIFT_URL_SUFFIX):
        container = container[:-len(su.SWIFT_URL_SUFFIX)]

    try:
        # First check the size
        headers = conn.head_object(container, obj)
        total_KB = int(headers.get('content-length', 0)) / 1024.0
        if total_KB > CONF.job_binary_max_KB:
            raise ex.DataTooBigException(
                round(total_KB, 1), CONF.job_binary_max_KB,
                _("Size of swift object (%(size)sKB) is greater "
                  "than maximum (%(maximum)sKB)"))

        headers, body = conn.get_object(container, obj)
    except swiftclient.ClientException as e:
        raise ex.SwiftClientException(six.text_type(e))

    return body


def _validate_job_binary_url(f):
    @functools.wraps(f)
    def wrapper(job_binary, *args, **kwargs):
        if not (job_binary.url.startswith(su.SWIFT_INTERNAL_PREFIX)):
            # This should have been guaranteed already,
            # but we'll check just in case.
            raise ex.BadJobBinaryException(
                _("Url for binary in internal swift must start with %s")
                % su.SWIFT_INTERNAL_PREFIX)
        names = _get_names_from_url(job_binary.url)
        if len(names) == 1:
            # a container has been requested, this is currently unsupported
            raise ex.BadJobBinaryException(
                _('Url for binary in internal swift must specify an object '
                  'not a container'))
        return f(job_binary, *args, **kwargs)
    return wrapper


@_validate_job_binary_url
def get_raw_data(job_binary, proxy_configs=None):
    conn_kwargs = {}
    if proxy_configs:
        conn_kwargs.update(username=proxy_configs.get('proxy_username'),
                           password=key_manager.get_secret(
                           proxy_configs.get('proxy_password')),
                           trust_id=proxy_configs.get('proxy_trust_id'))
    else:
        conn_kwargs.update(username=job_binary.extra.get('user'),
                           password=key_manager.get_secret(
                           job_binary.extra.get('password')))
    conn = sw.client(**conn_kwargs)
    return _get_raw_data(job_binary, conn)


@_validate_job_binary_url
def get_raw_data_with_context(job_binary):
    conn = sw.client_from_token()
    return _get_raw_data(job_binary, conn)



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_config import cfg
import six
from six.moves.urllib import parse as urlparse

from sahara.utils.openstack import base as clients_base


CONF = cfg.CONF

SWIFT_INTERNAL_PREFIX = "swift://"
SWIFT_URL_SUFFIX_START = '.'
SWIFT_URL_SUFFIX = SWIFT_URL_SUFFIX_START + 'sahara'


def retrieve_auth_url(endpoint_type="publicURL"):
    """This function returns auth url v3 api.

    """
    version_suffix = 'v3'

    # return auth url with trailing slash
    return clients_base.retrieve_auth_url(
        endpoint_type=endpoint_type, version=version_suffix) + "/"


def inject_swift_url_suffix(url):
    if isinstance(url, six.string_types) and url.startswith("swift://"):
        u = urlparse.urlparse(url)
        if not u.netloc.endswith(SWIFT_URL_SUFFIX):
            return url.replace(u.netloc,
                               u.netloc + SWIFT_URL_SUFFIX, 1)
    return url



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import xml.dom.minidom as xml

import six
import testtools

from sahara.utils import patches


class MinidomPatchesTest(testtools.TestCase):
    def setUp(self):
        super(MinidomPatchesTest, self).setUp()
        patches.patch_minidom_writexml()

    def _generate_n_prettify_xml(self):
        doc = xml.Document()
        pi = doc.createProcessingInstruction('xml-smth',
                                             'type="text/smth" '
                                             'href="test.smth"')
        doc.insertBefore(pi, doc.firstChild)
        configuration = doc.createElement("root")
        doc.appendChild(configuration)
        for idx in six.moves.xrange(0, 5):
            elem = doc.createElement("element")
            configuration.appendChild(elem)
            name = doc.createElement("name")
            elem.appendChild(name)
            name_text = doc.createTextNode("key-%s" % idx)
            name.appendChild(name_text)
            value = doc.createElement("value")
            elem.appendChild(value)
            value_text = doc.createTextNode("value-%s" % idx)
            value.appendChild(value_text)

        return doc.toprettyxml(indent="  ")

    def test_minidom_toprettyxml(self):
        self.assertEqual("""<?xml version="1.0" ?>
<?xml-smth type="text/smth" href="test.smth"?>
<root>
  <element>
    <name>key-0</name>
    <value>value-0</value>
  </element>
  <element>
    <name>key-1</name>
    <value>value-1</value>
  </element>
  <element>
    <name>key-2</name>
    <value>value-2</value>
  </element>
  <element>
    <name>key-3</name>
    <value>value-3</value>
  </element>
  <element>
    <name>key-4</name>
    <value>value-4</value>
  </element>
</root>
""", self._generate_n_prettify_xml())
